<?php

namespace App\Api\Http\Controllers;

use App\Admin\Dashboard\Models\Call;
use App\Admin\Dashboard\Models\CarEdit;
use App\Admin\Dashboard\Models\CarIncident;
use App\Admin\Dashboard\Models\CarModeration;
use App\Admin\Dashboard\Models\CarPublish;
use App\Admin\Dashboard\Models\CarReason;
use App\Admin\Dashboard\Models\Room;
use App\Admin\Dashboard\Models\Transaction;
use App\Admin\Dashboard\Models\Version;
use App\Auth\Models\UserType;
use App\Chat\Agent;
use App\Core\Http\Controllers\CoreController as CoreController;
use App\Core\Jobs\Car\CreateCarImageJob;
use App\Core\Jobs\CarStatusPodioJob;
use App\Core\Jobs\EditCarPodioJob;
use App\Core\Jobs\Fipe\AtualizaExternalidMarcaFipeJob;
use App\Core\Jobs\Fipe\AtualizaFipe;
use App\Core\Jobs\NewCarJob;
use App\Core\Jobs\NewCarPodioJob;
use App\Core\Jobs\NewSellPodioJob;
use App\Core\Jobs\SendEmailDenuncia;
use App\Core\Jobs\SendEmailSenha;
use App\Core\Jobs\SendPushDenunciaAdmin;
use App\Http\Requests\Car\CarSellRequest;
use App\Push\PushNotification;
use App\Repasse\Audit\CopyCar;
use App\Repasse\Audit\IsEditable;
use App\Repasse\CarUtil;
use App\Repasse\Fipe\FipeLimit;
use App\Repasse\Moderator\CheckDuplicated;
use App\Repasse\Moderator\CheckFipeValue;
use App\Repasse\Moderator\CheckIsUserFirstListing;
use App\Repasse\Moderator\CheckRepassador;
use App\Repasse\Moderator\CheckUserCreationDate;
use App\Repasse\Moderator\CheckUserType;
use App\Repasse\Search\SearchPopulate;
use App\Repasse\Services\ImageService;
use App\Repasse\TimeParser;
use Carbon\Carbon;
use GuzzleHttp\Client;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Pipeline\Pipeline;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Input;
use App\Auth\Models\User;
use Illuminate\Support\Facades\Auth;
use App\Admin\Dashboard\Models\Brand;
use App\Admin\Dashboard\Models\Optional;
use App\Admin\Dashboard\Models\CarPhoto;
use App\Admin\Dashboard\Models\CarSell;
use App\Admin\Dashboard\Models\CarIgnore;
use App\Admin\Dashboard\Models\CarOptional;
use App\Admin\Dashboard\Models\Car;
use App\Admin\Dashboard\Models\Config;
use App\Admin\Dashboard\Models\Color;
use App\Admin\Dashboard\Models\Fipe;
use DB;
use Image;
use Log;
use App\Traits\Rest\ResponseHelpers;
use App\Http\Requests\Car\CreateRequest;
use App\Http\Requests\Car\MigrateRequest;
use App\Transformers\Chat\ChatTransformer;
use App\Admin\Dashboard\Models\Store\Store;

class ApiController extends CoreController
{
    use DispatchesJobs, ResponseHelpers;

    const MODERATION_PIPES = [
        CheckUserType::class,
        CheckDuplicated::class,
        CheckFipeValue::class,
        CheckUserCreationDate::class,
        CheckRepassador::class,
        CheckIsUserFirstListing::class
    ];


    public function checkVersion()
    {

        $version = Version::where('current', 1)->get(['version', 'info'])->first();
        if ($version) {
            $html = '<div style="padding: 2%;">' . $version->info . '</div>';
            $data = ['version' => $version->version, 'info' => $html];
        } else {
            $data = ['version' => '0.0.1'];
        }
        return ['success' => true, 'data' => $data];
    }


    public function populateAnnounce($type = null)
    {

        $brands = Brand::where('type', $type)->orderBy('name', 'ASC')->get(['id', 'name']);
        $percent = Config::find(1);
        $optional = Optional::where('type', $type)->get(['id', 'name']);
        $years = range(1984, (date('Y') + 1));
        $years[] = 'Zero KM';
        $doors = range(2, 5);
        $eixos = array('1', '2', '3', '4', '4x2', '6x2', '6x4', '8x2', '8x4');
        $colors = Color::get(['id', 'name']);

        return array(
            'success' => true,
            'data' => array(
                'brands' => $brands,
                'percent' => $percent->percent,
                'optional' => $optional,
                'doors' => $doors,
                'eixos' => $eixos,
                'years' => $years,
                'colors' => $colors
            )
        );

    }

    public function populateYear($type, $brand_id)
    {
        $years = Fipe::where('brand_id', $brand_id)
            ->where('type', $type)
            ->groupBy('year')
            ->orderBy('year', 'DESC')
            ->get(['year'])->pluck('year');

        return ['success' => true, 'data' => $years];
    }

    /**
     * @SWG\Get(path="/populate-model/{brand_id}/{year}",
     *   tags={"Car"},
     *   summary="Popular a lista de modelos para o cadastro de anúncio com base na marca e no ano",
     *   description="",
     *   operationId="populateModel",
     *   produces={"application/json"},
     *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
     *    @SWG\Parameter( in="path", name="brand_id", description="", required=true, type="integer" ),
     *    @SWG\Parameter( in="path", name="year", description="", required=true, type="integer" ),
     *   @SWG\Response( response=200, description="",
     *       @SWG\Schema(
     *           @SWG\Property(property="success", type="boolean"),
     *           @SWG\Property(property="fipe_percent", type="integer"),
     *           @SWG\Property(property="data", type="array", @SWG\Items(
     *                       type="object", ref="#/definitions/Fipe"
     *           ))
     *       )
     *   ),
     *   @SWG\Response( response=400, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="error", type="string", description="Bad Request" )
     *       )
     *   )
     * )
     * @param null|integer $brand_id
     * @param null|integer $year
     * @return array|Response
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function populateModel($brand_id = null, $year = null)
    {

        $response = new Response();
        if (empty($brand_id) || empty($year)) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Marca ou ano não informados!']);
            return $response;
        }

        //$fipes = Fipe::where('brand_id', (int)$brand_id)->where('year', (int)$year)->orderBy('model')->get();

        $user = \auth()->user();
        $fipe_percent = FipeLimit::getFipePercent($user, $year);


        $modelosFipe = Fipe::where('external_fipe_id', '=', 0)
            ->where('brand_id', $brand_id)->get();

        /*
         * config e marca foram removidos da proteção do count de modelos.
         * por isso marca precisa de um try catch pra pegar retorno de nada encontrado.
         */
        $config = Config::find(1);
        try {
            $marca = Brand::findOrFail($brand_id);
        } catch (\Exception $e) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Marca não encontrada!']);
            return $response;
        }

        if ($modelosFipe->count() > 0) {
            $client = new Client();

            /*
             * Este verificador leva em consideração que todos os modelos em
             * $modelosFipe tenham uma marca com um nome e um external_fipe_id mesmo que seja 0
             * Sendo external_fipe_id === 0, compara com as marcas rovidas pela fipe e
             * atribui o external_fipe_id correto à marca.
             */
            if($marca->external_fipe_id === 0) {
                $request = $client->request('POST', config('fipe.api.url') . '/v1/marcas', ['json' => [
                    'codigo_tabela' => $config->current_fipe_table_id,
                    'tipo_veiculo'  => $marca->tipo->external_fipe_id
                ]]);

                $dadosMarcasFipe = json_decode($request->getBody());

                foreach ($dadosMarcasFipe as $key => $dadoMarcaFipe) {
                    if($dadoMarcaFipe->Label === $marca->name) {
                        $marca->external_fipe_id = $dadoMarcaFipe->Value;
                        $marca->save();
                    }
                }
            }
        }

        $job = (new AtualizaFipe($config, $marca, $brand_id, $year))->onQueue('Fipe');
        $this->dispatch($job);

        $fipes = Fipe::where('brand_id', (int)$brand_id)
            ->where('year', $year)
            ->orderBy('model')
            ->select(['id', 'modelo_id', 'brand_id', 'year', 'fuel', 'code', 'price', 'type',
                \DB::raw("CASE fuel WHEN 'Alcool' THEN CONCAT_WS(' ', model, fuel) ELSE model END AS model")])
            ->get();

        return ['success' => true, 'fipe_percent' => $fipe_percent, 'data' => $fipes];
    }

    public function populateFipe($fipe_id)
    {

        // esse endpoint recebe a fipe id do nosso banco e retorna a fipe DA FIPE
        // @TODO um tratamento pra quando a parada não tem external_fipe_id

        $fipe = Fipe::find($fipe_id);
        $config = Config::find(1);


        if ($fipe->external_fipe_id !== 0) {
            // ir na fipe buscar a fipe da fipe fipe
            $client = new Client();
            $response = $client->request('POST', config('fipe.api.url') . '/v1/veiculo', ['json' => [
                "codigo_tabela" => $config->current_fipe_table_id,
                "tipo_veiculo" => $fipe->tipo->external_fipe_id,
                "codigo_marca" => $fipe->brand->external_fipe_id,
                "codigo_modelo" => $fipe->external_fipe_id,
                "ano_combustivel" => "{$fipe->year}-{$fipe->fipe_combustivel}"
            ]]);

            $resultado = json_decode($response->getBody());

            // converter R$ 2.716,00 pra 2716,00
            $valor = str_replace('R$ ', '', $resultado->Valor);
            $fipe->price = str_replace(',', '.', str_replace('.', '', $valor));
            $fipe->save();

            return $fipe->toArray();
        }
    }
    

    /**
     * @SWG\Get(path="/populate-edit/{car_type}/{year}",
     *   tags={"Car"},
     *   summary="Popular a lista de informações necessárias para editar um anúncio com base na marca e no ano",
     *   description="",
     *   operationId="populateEdit",
     *   produces={"application/json"},
     *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
     *    @SWG\Parameter( in="path", name="car_type", description="", required=true,  type="integer" ),
     *    @SWG\Parameter( in="path", name="year",     description="", required=true, type="integer" ),
     *    @SWG\Response( response=200, description="",
     *        @SWG\Schema(
     *            @SWG\Property(property="success", type="boolean"),
     *            @SWG\Property(property="data", type="object",
     *                @SWG\Property(property="fipe", type="integer"),
     *                @SWG\Property(property="optional", type="array",
     *                    @SWG\Items(type="object", ref="#/definitions/CarOptional")
     *                ),
     *                @SWG\Property(property="doors", type="array",
     *                    @SWG\Items(type="integer")
     *                ),
     *                @SWG\Property(property="eixos", type="array",
     *                    @SWG\Items(type="string")
     *                ),
     *                @SWG\Property(property="colors", type="array",
     *                    @SWG\Items(type="object", ref="#/definitions/Color")
     *                ),
     *            ),
     *        ),
     *    ),
     *    @SWG\Response( response=400, description="",
     *        @SWG\Schema(
     *            @SWG\Property( property="error", type="string", description="Bad Request" )
     *        )
     *    )
     * )
     * @param $car_type
     * @param null $year
     * @return array
     */
    public function populateEdit($car_type, $year = null)
    {
        $user = \auth()->user();
        $fipe = FipeLimit::getFipePercent($user, $year);
        $optional = SearchPopulate::getOptionals($car_type);
        $doors = SearchPopulate::getDoors();
        $eixos = SearchPopulate::getEixos();
        $colors = SearchPopulate::getColors();

        return [
            'success' => true,
            'data' => [
                'fipe' => $fipe,
                'optional' => $optional,
                'doors' => $doors,
                'eixos' => $eixos,
                'colors' => $colors
            ]
        ];
    }

    public function populateReason()
    {
        $reasons = CarReason::select(['id as value', 'name as label'])->get();
        return ['success' => true, 'data' => $reasons];
    }

    public function myAds($user_id = null, $page = null)
    {
        $response = new Response();

        if (empty($user_id) OR empty($page)) {
            $response->setStatusCode(400);
            $response->setContent([
                'success' => false,
                'msg' => 'Código do usuário ou número da página não informado!'
            ]);
            return $response;
        }

        $user = Auth::user();
        if ($user->id <> $user_id) {
            $response->setStatusCode(403);
            $response->setContent([
                'success' => false,
                'msg' => 'O usuário informado é diferente do usuário autorizado'
            ]);
            return $response;
        }

        $data = Carbon::now();

        $cars = Car::where('user_id', $user_id)
            ->whereNotIn('car_status_id', [8,10])
            ->orderByRaw("FIELD(car_status_id, 6, 7, 4, 1, 2, 3)")
            ->orderBy('published_at', 'desc');

        $max = 15;
        $cars->limit($max)->offset(($page - 1) * $max);

        $cars = $cars->get();

        $ar_cars = array();

        foreach ($cars as $car) {
            $carObject = new CarUtil($car);
            $ar_cars[] = $carObject->getCarObject($user_id);
        }

        return array('success' => true, 'data' => $ar_cars);
    }

    public function purchase(Request $request)
    {

        $user = Auth::user();
        $user_id = $user->id;
        $user_type = $user->user_type;
        $page = $request->input('page');

        $response = new Response();

        if (empty($user_id) || empty($page)) {
            $response->setStatusCode(400);
            $response->setContent([
                'success' => false,
                'msg' => 'Código do usuário ou número da página não informado!'
            ]);
            return $response;
        }

        $query_profiles = '
        select
        p.*,
        c.car_type as car_type,
        s.ddd as DDD
        from user_profile p
        join user_profile_car_type c on c.profile_id = p.id
        join user_profile_ddd d on d.profile_id = p.id
        join state_ddds s ON s.id = d.ddd_id
        WHERE p.user_id = :user_id;
        ';

        $query_parameters = [
            'user_id' => $user_id
        ];

        $user_profiles = \DB::select($query_profiles, $query_parameters);

        $data = Carbon::now();

        $cars = Car::where('site_only', false)
            ->whereIn('car.car_status_id', [1, 2, 9])
            ->join('user', 'car.user_id', '=', 'user.id')
            ->join('car_publish', function ($join) use ($user_type) {
                $join->on('car.id', '=', 'car_publish.car_id')
                    ->on('car_publish.user_type', '=', DB::raw($user_type));
            })
            ->leftJoin('rooms', function ($join) use ($user_id) {
                $join->on('rooms.car_id', '=', 'car.id')
                    ->on('rooms.comprador_id', '=', DB::raw($user_id))
                    ->on('car.car_status_id', '=', DB::raw(2));
            })
            ->leftJoin('car_sell', function ($join) use ($user_id) {
                $join->on('car_sell.car_id', '=', 'car.id')
                    ->on('car_sell.user_buy_id', '=', DB::raw($user_id));
            })
            ->select('car.*')
            ->orderBy('rooms.id', 'DESC')->orderBy('car_sell.car_id', 'DESC')->orderBy('car.published_at', 'DESC')
            ->where(function ($query) use ($user_profiles) {

                foreach ($user_profiles as $profile) :
                    $query->orWhere(function ($query) use ($profile) {
                        $query->whereBetween('car.year', [$profile->start_year, $profile->end_year]);
                        $query->where('car.car_type', $profile->car_type);
                        $query->where('car.user_ddd', $profile->DDD);
                    });
                endforeach;

            })
            ->where('user.is_aprovado', 1)
            ->select('car.*')->distinct();

        $ignores = CarIgnore::where('user_id', $user_id)->lists('car_id');

        if ($ignores) {
            $cars->whereNotIn('car.id', $ignores->toArray());
        }

//        dd($cars->toSql() );

        $badges = clone $cars;
        $badges = $badges->whereIn('car.car_status_id', [1, 9]);

        if ($request->has('last_request')) {
            $badges = $badges->where('car.published_at', '>', $request->input('last_request'));
        }

        $badges = $badges->count();

        /*
         * Até aqui já lista a room_id se tiver negociando (e null se não tiver)
         * o que falta fazer é pegar o banco colocar esse usuário como negociando um anúncio e verificar como os resultados aparecem
         * e toda a parte do vendido :(
         * Agora falta toda a parte do veículo
         */

        $data = Carbon::now();


        $max = 15;
        $cars->limit($max)->offset(($page - 1) * $max);


        $cars = $cars->get();

        $ar_cars = array();

        foreach ($cars as $car) {
            unset($car->cost);
            $carUtil = new CarUtil($car);

            $ar_cars[] = $carUtil->getCarObject($user->id);

            //array_push($ar_cars, $ob_car);
        }


        return array(
            'success' => true,
            'last_request' => $request->input('last_request'),
            'badge' => $badges,
            'request' => $data->toRfc3339String(),
            'data' => $ar_cars
        );
    }


    /**
     *
     * @SWG\Post(path="/car",
     *   tags={"Car"},
     *   summary="Cria o anúncio",
     *   description="",
     *   operationId="carSave",
     *   produces={"application/json"},
     *   @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
     *   @SWG\Parameter( in="formData", name="color_id", description="ID da cor", required=true, type="integer" ),
     *   @SWG\Parameter( in="formData", name="doors", description="Portas (para carro e caminhão)", required=false, type="integer" ),
     *   @SWG\Parameter( in="formData", name="eixos", description="Eixos (para caminhão)", required=false, type="integer" ),
     *   @SWG\Parameter( in="formData", name="brand_id", description="ID da marca", required=true, type="integer" ),
     *   @SWG\Parameter( in="formData", name="fipe_id", description="ID da Fipe", required=true, type="integer" ),
     *   @SWG\Parameter( in="formData", name="year", description="Ano do modelo", required=true, type="integer" ),
     *   @SWG\Parameter( in="formData", name="observation", description="Observações", required=true, type="string" ),
     *   @SWG\Parameter( in="formData", name="cost", description="Custo do veículo", required=false, type="number" ),
     *   @SWG\Parameter( in="formData", name="price", description="Preço de venda do veículo", required=true, type="number" ),
     *   @SWG\Parameter( in="formData", name="km", description="Quilometragem do veículo", required=true, type="integer" ),
     *   @SWG\Parameter( in="formData", name="optionals", description="Opcionais", required=true, type="array" , @SWG\Items(type="integer")),
     *   @SWG\Response( response=200, description="", @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *            @SWG\Property( property="msg", type="string", description="Mensagem"),
     *            @SWG\Property( property="car_id", type="integer", description="Id do veículo cadastrado"),
     *        ) ),
     *   @SWG\Response( response=400, description="",
     *       @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *            @SWG\Property( property="error", type="string", description="Bad Request" )
     *       )
     *   ),
     *   @SWG\Response( response=401, description="",
     *       @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *            @SWG\Property( property="error", type="string", description="Null store" )
     *       )
     *   ),
     *   @SWG\Response( response=403, description="",
     *       @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *           @SWG\Property( property="error", type="string", description="Unauthorized" )
     *       )
     *   ),
     *   @SWG\Response( response=404, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="message", type="string", description="Model not found " ),
     *       )
     *   ),
     *
     *   @SWG\Response( response=409, description="",
     *       @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *           @SWG\Property( property="error", type="string", description="Fipe above limit" )
     *       )
     *   ),
     *   @SWG\Response( response=422, description="", @SWG\Schema(
     *            @SWG\Property( property="message", type="string", description="Validation failed" ),
     *            @SWG\Property( property="errors", type="array",
     *                @SWG\Items( type="object",
     *                    @SWG\Property(property="field", type="string"),
     *                )
     *            ),
     *        )
     *    ))
     *
     *
     * @param CreateRequest $request
     * @return array|\Illuminate\Http\JsonResponse
     */
    public function saveCar(CreateRequest $request)
    {

        $response = new Response();
        $user = Auth::user();

        if ($user->is_aprovado != 1) {
            return $this->badRequest([
                'success' => false,
                'msg' => trans('messages.error.unauthorized'),
                'errors' => []
            ], 403);
        }

        $params = $request->all();

        if ($user->hasRole(['chief', 'manager', 'salesman']) && !isset($params['store_id'])) {
            return $this->badRequest([
                'success' => false,
                'msg' => trans('messages.error.validation.store.null'),
                'errors' => []
            ], 422);
        }

        if (isset($params['store_id'])) {
            $belongs_store = $user->stores()->get()->contains(function ($key, $value) use ($params) {
                return $value->id == $params['store_id'];
            });
            if (!$belongs_store) {
                return $this->badRequest([
                    'success' => false,
                    'msg' => trans('messages.error.forbidden'),
                    'errors' => []
                ], 401);
            }
        }

        $car = new Car();
        $car->fill($params);
        //Ticket#78 O status do carro após o cadastro é id 4 (EM CADASTRO)
        $car->car_status_id = 4;
        // User id não é mais enviado
        $car->user_id = $user->id;
        $car->user_ddd = substr($user->cell_phone, 0, 2);

        // Ticket#103 salva o valor da fipe no carro
        $fipe = Fipe::find($params['fipe_id']);
        $car->fipe_price = $fipe->price;
        $car->car_type = $fipe->type;

        // Verificar valor da Fipe alterado para trazer o dado da configuração de fipe por tipo de usuário e ano
        //$config = Config::find(1);
        // $percent = -1 * abs($config->percent);
        $fipe_percent = FipeLimit::getDefaultFipePercent($user->user_type, $fipe->year);
        $percent = -1 * abs($fipe_percent);

        $fipe = ($params['price'] - $car->fipe_price) / $car->fipe_price * 100;

        $has_site = false;
        if ($fipe > $percent) {
            //
            $stores = $user->stores;

            foreach ($stores as $store) {
                if ($store->site) {
                    $has_site = true;
                }
            }

            if (!$has_site) {
                return $this->badRequest([
                    'success' => false,
                    'msg' => trans('messages.error.validation.fipe'),
                    'errors' => []
                ], 409);
            }
        }

        $car->site_only = $has_site;


        if ($car->save()) {

            foreach ($params['optionals'] as $optional_id) {
                $car_optional = new CarOptional();
                $car_optional->optional_id = $optional_id;
                $car_optional->car_id = $car->id;
                $car_optional->save();
            }

            $podioJob = (new NewCarPodioJob($car))->onQueue('push');
            $this->dispatch($podioJob);

            return array('success' => true, 'car_id' => $car->id, 'msg' => 'Anúncio cadastrado com sucesso');
        }

    }

    /**
     * @param Request $request
     * @return Response
     */
    public function publishCar(Request $request)
    {
        $usuario = Auth::user();
        $response = new Response();

        if (!$request->has('id')) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Falta um parâmetro']);
            return $response;
        }


        $id = $request->input('id');

        /** @var Car $car */
        $car = Car::find($id);
        if (!$car) {
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'Anúncio não encontrado!']);
            return $response;
        }

        if ($car->user_id != $usuario->id) {
            $response->setStatusCode(403);
            $response->setContent(['success' => false, 'msg' => 'Acesso negado']);
            return $response;
        }

        if ($car->car_status_id != 4) {
            $response->setStatusCode(412);
            $response->setContent(['success' => false, 'msg' => 'Anúncio já foi publicado']);
            return $response;
        }

        $quantidadeFotos = CarPhoto::where('car_id', $id)->get()->count();
        if (($quantidadeFotos < 4) OR ($quantidadeFotos > 10)) {
            $response->setStatusCode(409);
            $response->setContent([
                'success' => false,
                'msg' => 'A quantidade de fotos difere da solicitada (entre 4 e 10)'
            ]);
            return $response;
        }

        // Gera imagem de print
        ImageService::generateSaveImage($car);


        // Salva o carro e marca como em análise (5)
        $car->car_status_id = 5;

        $user_types = $request->get('user_types');
        if (!empty($user_types)) {
            foreach ($user_types as $user_type) {
                $carPublish = CarPublish::create(
                    [
                        'car_id' => $car->id,
                        'user_type' => $user_type
                    ]
                );
            }
        } else {
            $response->setStatusCode(400);
            $response->setContent([
                'success' => false,
                'msg' => 'Você precisa escolher um público para este anúncio'
            ]);
            return $response;
        }

        $car->save();

        $car_ob = new CarUtil($car);
        $car_data = $car_ob->getCarObject();

        $car_data->score = 10;
        $car_data->reasons = [];

        $anunciar = app(Pipeline::class)
            ->send($car_data)
            ->through(self::MODERATION_PIPES)
            ->then(function ($content) {
                Log::info('Finalizou rotina de moderação');

                // Checar score, se for abaixo de 10 manter status do anúncio
                if ($content->score <= 5) {
                    // Tabela de moderação.
                    $moderar = CarModeration::create([
                        'car_id' => $content->id,
                        'score' => $content->score,
                        'reasons' => $content->reasons
                    ]);
                }

                return $content;
            });

        // Se o anúncio é oferta exclusiva de site, ele não é publicado, apenas tem data de publicação
        if ($car->site_only) {
            $car->published_at = Carbon::now();
            $car->car_status_id = 1;
            $car->save();

            $response->setStatusCode(201);
            $response->setContent([
                'success' => true,
                'msg' => 'Oferta exclusiva do seu site criada com sucesso!',
                'car' => $car_data
            ]);

            return $response;
        }

        // Se o anúncio está no score aprovado ele pode ser publicado.
        if ($anunciar->score >= 10) {
            $job = (new NewCarJob($car_data))->onQueue("push");
            $this->dispatch($job);

            //

            /*$podioJob = (new NewCarPodioJob($car))->onQueue('push');
            $this->dispatch($podioJob);*/

            $msg = 'Anúncio publicado com sucesso';
        } else {
            $podioStatusJob = (new CarStatusPodioJob($car, 5))->onQueue('push');
            $this->dispatch($podioStatusJob);
            $msg = 'Anúncio encontra-se em análise';
        }
        $response->setStatusCode(201);
        $response->setContent(['success' => true, 'msg' => $msg, 'car' => $car_data]);
        return $response;


    }

    public function saveCarImage(Request $request, $car_id = null)
    {

        $response = new Response();

        // Checar se os parâmetros foram informados (car_id e filename)

        if (!isset($car_id)) {

            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Anúncio não informado']);
            return $response;
        }


        if ($request->hasFile('filename') AND $request->file('filename')->isValid()) {
            $car = Car::find($car_id);
            if (!$car) {
                $response->setStatusCode(404);
                $response->setContent(['success' => false, 'msg' => 'Anúncio não localizado']);
                return $response;
            }

            $usuario = Auth::user();

            if ($usuario->id != $car->user_id) {
                $response->setStatusCode(403);
                $response->setContent(['success' => false, 'msg' => 'Não autorizado']);
                return $response;
            }

            $audit = IsEditable::check($car->car_status_id);

            if ($audit['edit'] === false) {
                $response->setStatusCode(401);
                $response->setContent(['success' => false, 'msg' => 'Anúncio não pode ser editado']);
                return $response;
            }

            if ($audit['copy'] === true) {
                // Cópia do anúncio para todos os anúncios pode ser editados
                $carEdit = new CarEdit();
                $carUtil = new CarUtil($car);
                $copyCar = new CopyCar($carEdit, $usuario->id, $carUtil->getCarObject(), 3);
                $copyCar->make();
            }


            $mimes = array('image/jpeg' => 'jpg', 'image/png' => 'png');
            $file = $request->file('filename');
            $extension = $file->getClientOriginalExtension();
            $fileMime = $file->getClientMimeType();
            if (in_array($fileMime, array_keys($mimes))) {
                //Salvar arquivo
                $filename = uniqid() . time() . '.' . $extension;
                Storage::disk('anuncio-imagem')->put('original_' . $filename, File::get($file));
                //Rotacionar e canvas
                // $background = Image::canvas(600, 450, array(0,0,0));
                $image = Image::make($request->file('filename')->getRealPath())->orientate();
                $image->fit(600, 450);
                // $background->insert($image, 'center');
                Storage::disk('anuncio-imagem')->put($filename, $image->stream());
                $car_photo = CarPhoto::create(['filename' => $filename, 'car_id' => $car_id, 'photo_order' => $request->get('order')]);

                if ($car->car_status_id == 6) {
                    $car_status_id = 5;
                    $car->car_status_id = $car_status_id;
                    $car->is_edited = 1;
                    $car->save();
                }


                $data = [
                    'success' => true,
                    'data' => [
                        'id' => $car_photo->id,
                        'filename' => $car_photo->filename
                    ]
                ];
                return $data;


            } else {
                $response->setStatusCode(400);
                $response->setContent(['success' => false, 'msg' => 'Arquivo Inválido (Não é uma imagem jpeg ou png)']);
                return $response;
            }


        } else {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Arquivo não enviado ou inválido']);
            return $response;
        }
    }


    /**
     *
     * @SWG\Post(path="/car-edit",
     *   tags={"Car"},
     *   summary="Atualiza anúncio",
     *   description="",
     *   operationId="updateCar",
     *   produces={"application/json"},
     *   @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
     *   @SWG\Parameter( in="formData", name="car_id", description="ID do anúncio", required=true, type="integer" ),
     *   @SWG\Parameter( in="formData", name="color_id", description="ID da cor", required=true, type="integer" ),
     *   @SWG\Parameter( in="formData", name="doors", description="Portas (para carro e caminhão)", required=false, type="integer" ),
     *   @SWG\Parameter( in="formData", name="eixos", description="Eixos (para caminhão)", required=false, type="integer" ),
     *   @SWG\Parameter( in="formData", name="observation", description="Observações", required=true, type="string" ),
     *   @SWG\Parameter( in="formData", name="cost", description="Custo do veículo", required=false, type="number" ),
     *   @SWG\Parameter( in="formData", name="price", description="Preço de venda do veículo", required=true, type="number" ),
     *   @SWG\Parameter( in="formData", name="km", description="Quilometragem do veículo", required=true, type="integer" ),
     *   @SWG\Parameter( in="formData", name="optionals", description="Opcionais", required=true, type="array" , @SWG\Items(type="integer")),
     *   @SWG\Response( response=200, description="", @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *            @SWG\Property( property="msg", type="string", description="Mensagem"),
     *            @SWG\Property( property="car_id", type="integer", description="Id do veículo cadastrado"),
     *            @SWG\Property( property="data", type="object", @SWG\Schema(ref="#/definitions/Car"))
     *        ) ),
     *   @SWG\Response( response=400, description="",
     *       @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *            @SWG\Property( property="error", type="string", description="Bad Request" )
     *       )
     *   ),
     *   @SWG\Response( response=401, description="",
     *       @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *            @SWG\Property( property="error", type="string", description="Null store" )
     *       )
     *   ),
     *   @SWG\Response( response=403, description="",
     *       @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *           @SWG\Property( property="error", type="string", description="Unauthorized" )
     *       )
     *   ),
     *   @SWG\Response( response=404, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="message", type="string", description="Model not found " ),
     *       )
     *   ),
     *
     *   @SWG\Response( response=409, description="",
     *       @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *           @SWG\Property( property="error", type="string", description="Fipe above limit" )
     *       )
     *   ),
     *   @SWG\Response( response=422, description="", @SWG\Schema(
     *            @SWG\Property( property="message", type="string", description="Validation failed" ),
     *            @SWG\Property( property="errors", type="array",
     *                @SWG\Items( type="object",
     *                    @SWG\Property(property="field", type="string"),
     *                )
     *            ),
     *        )
     *    ))
     *
     * @param Request $request
     * @return array|Response
     * @throws \InvalidArgumentException
     */
    public function updateCar(Request $request)
    {

        $response = new Response();

        /** @var Car $car */
        $car = Car::findOrFail($request->get('car_id'));

        $user = Auth::user();
        if (($user->is_aprovado != 1) || ($car->user_id <> $user->id)) {
            $response->setStatusCode(403);
            $response->setContent(['success' => false, 'msg' => 'Usuário não autorizado']);
            return $response;
        }

        $audit = IsEditable::check($car->car_status_id);

        if ($audit['edit'] === false) {
            $response->setStatusCode(401);
            $response->setContent(['success' => false, 'msg' => 'Anúncio não pode ser editado']);
            return $response;
        }

        if ($audit['copy'] === true) {
            // Cópia do anúncio para todos os anúncios pode ser editados
            $carEdit = new CarEdit();
            $carUtil = new CarUtil($car);
            $copyCar = new CopyCar($carEdit, $user->id, $carUtil->getCarObject(), 3);
            $copyCar->make();
        }

        $params = Input::all();

        if (!empty($params)) {
            $validation = Validator::make(
                $params,
                [
                    'color_id' => ['required'],
                    'observation' => ['required'],
                    'price' => ['required'],
                    'km' => ['required']
                ],
                [
                    'required' => 'O campo precisa ser preenchido.',
                ]
            );

            if ($validation->fails()) {
                $response->setStatusCode(400);
                $response->setContent(['success' => false, 'msg' => $validation->messages()]);
                return $response;
            }

            // Atualizar campos (color_id, doors, preço, km)
            $car->color_id = $request->get('color_id');
            $car->doors = $request->get('doors', '');
            $car->eixos = $request->get('eixos', '');
            $car->price = $request->get('price');

            if ($request->has('cost')) {
                $car->cost = $request->get('cost');
            }

            $car->km = $request->get('km', '');
            $car->observation = $request->get('observation', '');
            // Ticket#458 - Salvar o preço novo da fipe
            $fipe = Fipe::find($car->fipe_id);
            $car->fipe_price = $fipe->price;

            // Conforme combinado, editou (mas não estava em cadastro) vai pra análise
            if ($car->car_status_id == 6) {
                $car_status_id = 5;
                $car->car_status_id = $car_status_id;
                $car->is_edited = 1;
            }

            // Atualizar com os dados o cadastro
            $car->user_ddd = substr($user->cell_phone, 0, 2);

            if ($car->save()) {
                if (isset($params['optionals'])) {
                    // deleta opcionais antigos
                    $car->optionals()->delete();

                    // cadastra os novos
                    foreach ($params['optionals'] as $optional_id) {
                        $car_optional = new CarOptional();
                        $car_optional->optional_id = $optional_id;
                        $car_optional->car_id = $car->id;
                        $car_optional->save();
                    }

                }
                $carUtil = new CarUtil($car);

                $carEditJob = (new EditCarPodioJob($car))->onQueue('push');
                $this->dispatch($carEditJob);

                $printJob = (new CreateCarImageJob($car))->onQueue('push');
                $this->dispatch($printJob);

                return array(
                    'success' => true,
                    'car_id' => $car->id,
                    'msg' => 'Anúncio alterado com sucesso',
                    'data' => $carUtil->getCarObject()
                );
            }
        }
    }


    /**
     * Remove um carro do app
     * @param Request $request
     * @return Response
     * @throws \InvalidArgumentException
     */
    public function removeCar(Request $request)
    {
        $response = new Response();
        $user = Auth::user();

        if ($request->has('id')) {
            $carros_id = $request->input('id');

            $veiculo = Car::find($carros_id);

            // Verifica se o usuário é dono do anúncio
            if ($veiculo->user_id != $user->id) {
                $response->setStatusCode(403);
                $response->setContent(['success' => false, 'msg' => 'Acesso negado']);
                return $response;
            }
            // Verifica se o anúncio está em aberto (vendido não pode ser excluído)
            if ($veiculo->car_status_id == 3) {
                $response->setStatusCode(409);
                $response->setContent(['success' => false, 'msg' => 'Um ou mais veículos estão vendidos']);
                return $response;
            }

            if ($veiculo->car_status_id == 4) {
                // Veículo não foi publicado e será excluído
                // Tarefa#822 - Veículo em cadastro passa a ser soft-deletado igual aos outros
                $veiculo->car_status_id = 8;
                $veiculo->save();
                $podioJob = (new CarStatusPodioJob($veiculo, 8))->onQueue('push');
                $this->dispatch($podioJob);

                $response->setStatusCode(200);
                $response->setContent(['success' => true, 'msg' => 'Veículo removido com sucesso']);
                return $response;

            } else {
                // Carro está em aberto/análise/suspenso e tem que bloquear o chat enquanto marca o carro como excluído
                $car_status_id = 8;
                $veiculo->car_status_id = $car_status_id;
                $veiculo->save();
                $host = config('chat.host');
                $port = config('chat.port');
                $token = config('chat.token');
                $message = [
                    'sender_id' => $veiculo->user_id,
                    'receiver_id' => null,
                    'car_id' => $veiculo->id,
                    'car_status_id' => $car_status_id
                ];

                $agent = new Agent($host, $port, $token);
                $agent->sendMessage($message);

                $sell = new CarSell();
                $sell->car_id = $veiculo->id;
                $sell->user_sell_id = $veiculo->user_id;
                $sell->user_buy_id = 0;
                $sell->reason_id = 3;
                $sell->save();

                $podioJob = (new CarStatusPodioJob($veiculo, $car_status_id))->onQueue('push');
                $this->dispatch($podioJob);

                $response->setStatusCode(200);
                $response->setContent(['success' => true, 'msg' => 'Veículo removido com sucesso']);
                return $response;

            }
        } else {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Faltam parâmetros para completar a requisição']);
            return $response;
        }
    }

    /**
     * @param Request $request
     * @return Response
     */
    public function ignoreCar(Request $request)
    {

        $user = Auth::user();
        $response = new Response();

        if ($request->has('id')) {

            $ignore_id = $request->input('id');

            $ignore_results = array();
            foreach ($ignore_id as $id) {
                $ignore = new CarIgnore();
                $ignore->car_id = $id;
                $ignore->user_id = $user->id;
                if ($ignore->save()) {
                    $ignore_results[] = $ignore->id;
                }
            }

            $response->setStatusCode(200);
            $response->setContent(['success' => true, 'data' => $ignore_results]);
            // return $response;

        } else {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Faltam parâmetros para completar a solicitação']);
            // return $response;
        }

        return $response;

    }


    /**
     *
     * @SWG\Post(path="/car-sold",
     *   tags={"Car"},
     *   summary="Sell the car",
     *   description="",
     *   operationId="carSold",
     *   produces={"application/json"},
     *   @SWG\Parameter( in="formData", name="car_id", description="ID do Veículo", required=true, type="integer", @SWG\Schema(ref="#/definitions/Product") ),
     *   @SWG\Parameter( in="formData", name="reason_id", description="ID do tipo de finalização", required=true, type="integer", @SWG\Schema(ref="#/definitions/CarReason") ),
     *   @SWG\Parameter( in="formData", name="buyer_id", description="ID do usuário comprador", required=false, type="string", @SWG\Schema(ref="#/definitions/User") ),
     *   @SWG\Response( response=200, description="", @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Status da transação" ),
     *            @SWG\Property( property="msg", type="string", description="Mensagem"),
     *            @SWG\Property( property="car_status_id", type="integer", description="Situação atual do carro vendido=3, excluído=8"),
     *        ) ),
     *   @SWG\Response( response=400, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="error", type="string", description="Bad Request" )
     *       )
     *   ),
     *   @SWG\Response( response=404, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="message", type="string", description="Model not found " ),
     *       )
     *   ),
     *   @SWG\Response( response=422, description="", @SWG\Schema(
     *            @SWG\Property( property="message", type="string", description="Validation failed" ),
     *            @SWG\Property( property="errors", type="array",
     *                @SWG\Items( type="object",
     *                    @SWG\Property(property="field", type="string"),
     *                )
     *            ),
     *        )
     *    ),
     * )
     * @param CarSellRequest $request
     * @return Response
     * @throws \Throwable
     */
    public function carSold(CarSellRequest $request)
    {
        $response = new Response();
        // $vendedor = Auth::user();
        $vendedor = auth()->user();

        $car_id = $request->get('car_id');
        $comprador_id = $request->get('buyer_id', 0);
        // Foram encontrados casos em que o usuário envia o buyer_id como null fazendo a linha de cima praticamente retornar null ao invés de zero
        $comprador_id = $comprador_id ?? 0;

        $motivo = $request->get('reason_id');


        $car = Car::find($car_id);

        // Carro não encontrado ou excluído
        if (empty($car) or $car->car_status_id == 8) {
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'Carro não encontrado']);
            return $response;
        }

        if ($car->user_id != $vendedor->id) {
            $response->setStatusCode(403);
            $response->setContent(['success' => false, 'msg' => 'O usuário não é o proprietário do carro']);
            return $response;
        }

        // Carro só pode ser marcado como vendido ou excluído se estiver aberto (status 1 e 2)
        if (in_array($car->car_status_id, [1, 2, 5, 6, 7, 9], true)) {
            $host = config('chat.host');
            $port = config('chat.port');
            $token = config('chat.token');
            $message = [
                'sender_id' => $car->user_id,
                'receiver_id' => null,
                'car_id' => $car->id,
                'car_status_id' => null

            ];
            $agent = new Agent($host, $port, $token);
            // Motivos que precisam de usuário comprador: 1
            if ($motivo == 1) {
                // Vendeu o carro pelo aplicativo repasse
                $car_status_id = 3;

                $car->car_status_id = $car_status_id;
                $sell = new CarSell();
                $sell->car_id = $car->id;
                $sell->user_sell_id = $car->user_id;
                $sell->user_buy_id = $comprador_id;
                $sell->reason_id = $motivo;


                DB::transaction(function () use ($car, $sell, $comprador_id, $vendedor) {
                    $car->save();
                    $sell->save();

                    // Transação
                    if ($car->store_id) {
                        $store_id = $car->store_id;
                    } else {
                        // se o usuário tem lojas, a primeira em que ele é ativo. mystore não serve pois ele não é diretor da loja
                        $store_id = $vendedor->stores()->where('store_user.status', 1)->first()->id;
                    }

                    $seller = Transaction::create([
                        'type' => 1,
                        'car_id' => $car->id,
                        'user_id' => $car->user_id,
                        'store_id' => $store_id
                    ]);
                    if ($comprador_id !== 0) {
                        //
                        $comprador = User::find($comprador_id);
                        $buyer = Transaction::create([
                            'type' => 2,
                            'car_id' => $car->id,
                            'user_id' => $comprador_id,
                            'store_id' => $comprador->stores()->where('store_user.status', 1)->first()->id
                        ]);
                    }
                });
                // Podio
                $podioJob = (new NewSellPodioJob($sell))->onQueue('push');
                $this->dispatch($podioJob);

                // Mensagem
                $message['car_status_id'] = $car_status_id;
                $message['car_buyer'] = $comprador_id;

                $response->setStatusCode(200);
                $response->setContent([
                    'success' => true,
                    'msg' => 'Parabéns, seu veículo foi vendido.',
                    'car_status_id' => $car->car_status_id
                ]);
            }

            if ($motivo == 2) {
                // Vendeu o carro por outros meios
                $car_status_id = 3;

                $sell = new CarSell();
                $sell->car_id = $car->id;
                $sell->user_sell_id = $car->user_id;
                $sell->user_buy_id = $comprador_id;
                $sell->reason_id = $motivo;
                $car->car_status_id = $car_status_id;

                DB::transaction(function () use ($car, $sell) {
                    $car->save();
                    $sell->save();
                });

                $message['car_status_id'] = $car_status_id;

                $response->setStatusCode(200);
                $response->setContent([
                    'success' => true,
                    'msg' => 'Parabéns, seu veículo foi vendido.',
                    'car_status_id' => $car->car_status_id
                ]);
            }

            if ($motivo == 3) {
                // Desistiu de vender
                $car_status_id = 8;
                $car->car_status_id = $car_status_id;

                $sell = new CarSell();
                $sell->car_id = $car->id;
                $sell->user_sell_id = $car->user_id;
                $sell->user_buy_id = 0;
                $sell->reason_id = $motivo;
                $car->car_status_id = $car_status_id;

                DB::transaction(function () use ($car, $sell) {
                    $car->save();
                    $sell->save();
                });

                $message['car_status_id'] = $car_status_id;

                $response->setStatusCode(200);
                $response->setContent([
                    'success' => true,
                    'msg' => 'Veículo removido com sucesso',
                    'car_status_id' => $car->car_status_id
                ]);
            }

            $podioStatusJob = (new CarStatusPodioJob($car, $car_status_id))->onQueue('push');
            $this->dispatch($podioStatusJob);

            $agent->sendMessage($message);

            return $response;
        } else {
            $response->setStatusCode(409);
            $response->setContent(array(
                'success' => false,
                'msg' => 'Veículo não pode ser marcado como vendido pois seu status não está aberto'
            ));
            return $response;
        }

    }

    public function carNegociando($id = null)
    {
        /*
         * validar dados obrigatóorios (id) e retornar 400 caso não tenham sido informados OK
         * verificar se o carro existe, retornando 404 caso não exista OK
         * verificar se o carro é do usuário que fez a solicitação OK
         */

        $user = Auth::user();
        $response = new Response();

        if (empty($id)) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'O campo id não foi informado']);
            return $response;
        }

        $car = Car::find($id);

        if ($car) {

            if ($car->user_id != $user->id) {
                $response->setStatusCode(403);
                $response->setContent([
                    'success' => false,
                    'msg' => 'O anúncio que você está tentando alterar não é seu'
                ]);
                return $response;
            } else {
                //inverte o sinal, sério dub?
                $status = ($car->car_status_id != 2) ? 2 : 1;
                $car->car_status_id = $status;
                $car->save();

                $podioJob = (new CarStatusPodioJob($car, 2))->onQueue('push');
                $this->dispatch($podioJob);

                return array('success' => true, 'data' => ['status' => $car->car_status_id]);
            }

        } else {
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'Veículo não encontrado']);
            return $response;
        }

    }

    public function removeCarPhoto($id = null)
    {
        $response = new Response();
        $usuario = Auth::user();

        if (!$id) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Falta um parâmetro']);
            return $response;
        }

        $car_photo = CarPhoto::find($id);

        if (!$car_photo) {
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'Imagem não encontrada']);
            return $response;
        }

        $car = Car::findOrFail($car_photo->car_id);


        $audit = IsEditable::check($car->car_status_id);

        if ($audit['edit'] === false) {
            $response->setStatusCode(401);
            $response->setContent(['success' => false, 'msg' => 'Anúncio não pode ser editado']);
            return $response;
        }

        if ($audit['copy'] === true) {
            // Cópia do anúncio para todos os anúncios pode ser editados
            $carEdit = new CarEdit();
            $carUtil = new CarUtil($car);
            $copyCar = new CopyCar($carEdit, $usuario->id, $carUtil->getCarObject(), 3);
            $copyCar->make();
        }

        if ($usuario->id != $car->user_id) {
            $response->setStatusCode(403);
            $response->setContent(['success' => false, 'msg' => 'Acesso negado']);
            return $response;
        }

        $filename = $car_photo->filename;

        if ($car_photo->delete()) {
            Storage::disk('anuncio-imagem')->delete('original_' . $filename);

            if ($car->car_status_id == 6) {
                $car->car_status_id = 5;
                $car->is_edited = 1;
                $car->save();
            }

            return array('success' => true);
        }
    }

    public function chatUploadImage(Request $request)
    {
        $response = new Response();

        if ($request->hasFile('filename') AND $request->file('filename')->isValid()) {
            $mimes = array('image/jpeg' => 'jpg', 'image/png' => 'png');
            $file = $request->file('filename');
            $extension = $file->getClientOriginalExtension();
            $fileMime = $file->getClientMimeType();
            if (in_array($fileMime, array_keys($mimes))) {

                $name = uniqid() . time();
                $filename = $name . '.' . $extension;
                Storage::disk('chat-images')->put($filename, File::get($file));

                $image = Image::make($request->file('filename')->getRealPath())->orientate();
                $nova = $image->fit(200, 200)->pixelate(12)->encode('jpg', 1);

                Storage::disk('chat-images')->put('thumb_' . $name . '.jpg', $nova->stream());

                return ['success' => true, 'filename' => $filename];

            } else {
                $response->setStatusCode(400);
                $response->setContent(['success' => false, 'msg' => 'Tipo de arquivo inválido']);
                return $response;
            }

        } else {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Arquivo não enviado ou conteúdo inválido']);
            return $response;
        }

    }


    public function chatImage($filename)
    {

        try {
            $image = Storage::disk('chat-images')->get($filename);
        } catch (\Illuminate\Contracts\Filesystem\FileNotFoundException $fl) {
            $response = new Response();
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'Arquivo não encontrado']);
            return $response;
        }

        return Image::make($image)->response();

    }

    public function chatUploadAudio(Request $request)
    {
        $response = new Response();

        if ($request->hasFile('filename') AND $request->file('filename')->isValid()) {
            $filename = uniqid() . time() . '.m4a';
            $file = $request->file('filename');
            Storage::disk('chat-audios')->put($filename, File::get($file));
            return ['success' => true, 'filename' => $filename];

        } else {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Arquivo não enviado ou com problema']);
            return $response;
        }

    }

    public function chatAudio($filename)
    {
        try {
            $audio = Storage::disk('chat-audios')->get($filename);
        } catch (\Illuminate\Contracts\Filesystem\FileNotFoundException $fl) {
            $response = new Response();
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'Arquivo não encontrado']);
            return $response;
        }

        $response = new Response($audio);

        return $response;

    }


    public function infoChat($car_id = null, $client_id = null)
    {
        $response = new Response();
        if (is_null($car_id) OR is_null($client_id)) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Parâmetro obrigatório não informado']);
            return $response;
        }

        $car = Car::find($car_id);
        if ($car == null) {
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'Anúncio informado não encontrado']);
            return $response;
        }
        $client = User::find($client_id);
        if (!$client) {
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'Usuário informado não encontrado']);
            return $response;
        }

        // Usuário autenticado
        $user = Auth::user();

        $carUtil = new CarUtil($car);
        $ob_car = $carUtil->getCarObject($user->id);

        if ($car->car_status_id == 3) {
            $car_sell = CarSell::where('car_id', $car->id)->first();

            $ob_car->buyer_id = $car_sell->user_buy_id;
        }
        $user_type = UserType::find($client->user_type);
        $client->type = ($user_type ? $user_type->toArray() : ['id' => 0, 'name' => 'Não Informado']);

        return array('success' => true, 'data' => ['car' => $ob_car, 'client' => $client]);
    }

    /**
     * @SWG\Get(path="/store/{store_id}/car/{car_id}/chats",
     *   tags={"Chat"},
     *   summary="Listar todas as salas de chat de um anuncio",
     *   description="",
     *   operationId="listChatsByCar",
     *   produces={"application/json"},
     *   @SWG\Parameter( in="query", name="per_page", description="Default: 20", required=false, type="integer" ),
     *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
     *    @SWG\Parameter( in="path", name="car_id", description="", required=true, type="integer" ),
     *    @SWG\Parameter( in="path", name="store_id", description="", required=true, type="integer" ),
     *   @SWG\Response( response=200, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="status_code", type="integer", description="200" ),
     *           @SWG\Property( property="data", type="array",
     *               @SWG\Items( type="object",
     *                   @SWG\Property(property="totals", type="integer"),
     *                   @SWG\Property(property="per_page", type="integer"),
     *                   @SWG\Property(property="current_page", type="integer"),
     *                   @SWG\Property(property="badge", type="integer"),
     *                   @SWG\Property(property="items", type="array", @SWG\Items(
     *                       type="object", ref="#/definitions/Room"
     *                   ))
     *               )
     *           ),
     *       )
     *   ),
     *   @SWG\Response( response=400, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="error", type="string", description="Bad Request" )
     *       )
     *   )
     * )
     */
    public function getAllChats($store_id, $car_id)
    {
        try {
            $user = auth()->user();

            if (!$store = Store::find($store_id)) {
                return $this->modelNotFound([
                    'success' => false,
                    'msg' => trans('messages.error.notfound'),
                    'errors' => []
                ]);
            }

            if (!$car = Car::where('store_id', $store->id)->whereId($car_id)->first()) {
                return $this->modelNotFound([
                    'success' => false,
                    'msg' => trans('messages.error.notfound'),
                    'errors' => []
                ]);
            }

            $belongs_store = $store->staff->contains(function ($key, $value) use ($user) {
                return $value->id == $user->id;
            });

            if (!$belongs_store && !$user->hasRole('admin')) {
                return $this->badRequest([
                    'success' => false,
                    'msg' => trans('messages.error.forbidden'),
                    'errors' => []
                ], 401);
            }

            if (!$user->hasRole(['manager', 'chief', 'admin']) && $car->user_id != $user->id) {
                return $this->badRequest([
                    'success' => false,
                    'msg' => trans('messages.error.forbidden'),
                    'errors' => []
                ], 401);
            }

            return (new ChatTransformer)->transformCollection($car->rooms, $car->user_id);
        } catch (\Exception $e) {
            \Sentry::captureException($e);
            Log::error($e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine());
            return $this->badRequest(['success' => false, 'msg' => trans('messages.error.unavailable'), 'errors' => []],
                500);
        }
    }

    /**
     * @SWG\Get(path="/car/{car_id}/chat/{chat_id}",
     *   tags={"Chat"},
     *   summary="Visualizar um chat pelo ID da sala",
     *   description="",
     *   operationId="viewChat",
     *   produces={"application/json"},
     *   @SWG\Parameter( in="query", name="per_page", description="Default: 20", required=false, type="integer" ),
     *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
     *    @SWG\Parameter( in="path", name="car_id", description="", required=true, type="integer" ),
     *    @SWG\Parameter( in="path", name="chat_id", description="", required=true, type="integer" ),
     *   @SWG\Response( response=200, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="status_code", type="integer", description="200" ),
     *           @SWG\Property( property="data", type="array",
     *               @SWG\Items( type="object",
     *                   @SWG\Property(property="totals", type="integer"),
     *                   @SWG\Property(property="per_page", type="integer"),
     *                   @SWG\Property(property="current_page", type="integer"),
     *                   @SWG\Property(property="items", type="array", @SWG\Items(
     *                       type="object", ref="#/definitions/Room"
     *                   ))
     *               )
     *           ),
     *       )
     *   ),
     *   @SWG\Response( response=400, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="error", type="string", description="Bad Request" )
     *       )
     *   )
     * )
     */
    public function viewChat($car_id, $room_id)
    {
        try {
            $user = auth()->user();

            if (!$room = Room::find($room_id)) {
                return $this->modelNotFound([
                    'success' => false,
                    'msg' => trans('messages.error.notfound'),
                    'errors' => []
                ]);
            }

            // if ( !$car = Car::whereId( $car_id )->first() ) {
            //     return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            // }

            // $belongs_store = $car->store->staff->contains( function( $key, $value ) use( $user ) {
            //     return $value->id == $user->id;
            // });

            // if ( !$belongs_store && !$user->hasRole( 'admin' ) ) {
            //     return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            // }

            // if ( !$user->hasRole( ['manager', 'chief', 'admin'] ) && $car->user_id != $user->id ) {
            //     return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            // }

            return (new ChatTransformer)->transform($room);
        } catch (\Exception $e) {
            \Sentry::captureException($e);
        }
    }


    /**
     * @param null $car_id
     * @return array|\Illuminate\Http\JsonResponse|Response
     *
     * @SWG\Get(path="/car-negotiations/{car_id}",
     *   tags={"Car"},
     *   summary="Negociações de um anúncio",
     *   description="",
     *   operationId="carNegotiations",
     *   produces={"application/json"},
     *   @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
     *   @SWG\Parameter( in="path", name="car_id", description="", required=true, type="integer" ),
     *   @SWG\Response( response=200, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="success", type="boolean" ),
     *           @SWG\Property( property="data", type="array",
     *               @SWG\Items( type="object",
     *                   @SWG\Property(property="room_id", type="integer"),
     *                   @SWG\Property(property="car_id", type="integer"),
     *                   @SWG\Property(property="comprador_id", type="integer"),
     *                   @SWG\Property(property="vendedor_id", type="integer"),
     *                   @SWG\Property(property="bid", type="double"),
     *                   @SWG\Property(property="created_at", type="string"),
     *                   @SWG\Property(property="message", type="string"),
     *                   @SWG\Property(property="audio", type="string"),
     *                   @SWG\Property(property="image", type="string"),
     *                   @SWG\Property(property="chat", type="boolean"),
     *                   @SWG\Property(property="call", type="boolean"),
     *                   @SWG\Property(property="client", type="object", ref="#/definitions/User")
     *               )
     *           ),
     *       )
     *   ),
     *   @SWG\Response( response=401, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="error", type="string", description="Forbidden" )
     *       )
     *   ),
     *  @SWG\Response( response=404, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="error", type="string", description="Not found" )
     *       )
     *   ),
     * )
     */
    public function carNegotiations($car_id)
    {
        try {
            $user = auth()->user();
            if (!$car = Car::find($car_id)) {
                return $this->modelNotFound([
                    'success' => false,
                    'msg' => trans('messages.error.notfound'),
                    'errors' => []
                ]);
            }

            $belongs_store = $car->store->staff->contains(function ($key, $value) use ($user) {
                return $value->id == $user->id;
            });

            if (!$belongs_store && !$user->hasRole('admin') && $car->user_id != $user->id) {
                return $this->badRequest([
                    'success' => false,
                    'msg' => trans('messages.error.forbidden'),
                    'errors' => []
                ], 401);
            }

            if (!$user->hasRole(['manager', 'chief']) && $car->user_id != $user->id) {
                return $this->badRequest([
                    'success' => false,
                    'msg' => trans('messages.error.forbidden'),
                    'errors' => []
                ], 401);
            }

            /*
             * Alteração da tarefa #924 as negociações aparecem por ordem de LANCE
             */
            $query = '
                SELECT
                    r.id as room_id,
                    r.car_id,
                    r.comprador_id,
                    c.user_id as vendedor_id,
                    r.bid,
                    r.created_at
                FROM rooms r
                JOIN car c ON c.id = r.car_id
                WHERE c.id = :carid
                ORDER BY r.bid DESC
                ';

            $queryParameters = [
                'carid' => $car_id
            ];
            $chats = \DB::select($query, $queryParameters);
            $car_sell = CarSell::where('car_id', $car_id)->first();

            $ar_chats = array();

            foreach ($chats as $chat) {
                $ob_chat = (object)$chat;

                $ob_chat->message = null;
                $ob_chat->audio = null;
                $ob_chat->image = null;
                $ob_chat->chat = true;
                $ob_chat->call = false;
                $ob_chat->created_at = TimeParser::format($ob_chat->created_at);
                $ob_chat->client = (object)User::with('type')->find($ob_chat->comprador_id)->toArray();

                if (isset($car_sell->user_buy_id)) {
                    if ($car_sell->user_buy_id == $ob_chat->comprador_id) {
                        $ob_chat->client->is_comprador = true;
                    } else {
                        $ob_chat->client->is_comprador = false;
                    }
                }
                $ar_chats[] = $ob_chat;
            }

            // Consulta
            $queryc = 'SELECT
                r.id as room_id,
                r.car_id,
                r.comprador_id,
                c.user_id as vendedor_id,

                r.created_at
                FROM calls r
                JOIN car c ON c.id = r.car_id
                WHERE c.id = :carid
                ORDER BY r.id DESC';

            $calls = \DB::select($queryc, $queryParameters);

            foreach ($calls as $call) {

                $ob_call = (object)$call;
                $ob_call->message = null;
                $ob_call->audio = null;
                $ob_call->image = null;
                $ob_call->call = true;
                $ob_call->created_at = TimeParser::format($ob_call->created_at);
                $ob_call->client = (object)User::with('type')->find($ob_call->comprador_id)->toArray();


                if (isset($car_sell->user_buy_id)) {
                    if ($car_sell->user_buy_id == $ob_call->comprador_id) {
                        $ob_call->client->is_comprador = true;
                    } else {
                        $ob_call->client->is_comprador = false;
                    }
                }

                $ar_chats[] = $ob_call;
            }

            return array('success' => true, 'data' => $ar_chats);

        } catch (\Exception $e) {
            \Sentry::captureException($e);
            Log::error($e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine());
            return $this->badRequest(['success' => false, 'msg' => trans('messages.error.unavailable'), 'errors' => []],
                500);
        }
    }

    public function negotiations($car_id = null)
    {
        $response = new Response();

        if (empty($car_id)) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Parâmetro car_id não informado']);
            return $response;
        }


        $car = Car::find($car_id);


        if (!$car) {
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'Anúncio não encontrado']);
            return $response;
        }

        $user = Auth::user();
        if ($user->id != $car->user_id) {
            $response->setStatusCode(403);
            $response->setContent(['success' => false, 'msg' => 'Anúncio não pertence ao usuário']);
            return $response;

        }

        $rooms = Room::where('car_id', $car_id)->get(['comprador_id'])->pluck('comprador_id');

        $negotiations = User::whereIn('id', $rooms)->get(['id', 'name', 'photo']);

        return ['success' => true, 'data' => $negotiations];

    }


    /**
     * @SWG\Post(
     *     path="/call",
     *     tags={"Calls"},
     *     summary="Criação de uma nova chamada",
     *     description="Este endpoint cria uma nova chamada com os parâmetros fornecidos",
     *     @SWG\Parameter(
     *         name="body",
     *         in="body",
     *         required=true,
     *         @SWG\Schema(
     *             type="object",
     *             @SWG\Property(
     *                 property="car_id",
     *                 type="integer",
     *                 description="ID do carro"
     *             ),
     *             @SWG\Property(
     *                 property="comprador_id",
     *                 type="integer",
     *                 description="ID do comprador"
     *             )
     *         )
     *     ),
     *     @SWG\Response(
     *         response=200,
     *         description="Resposta de sucesso",
     *         @SWG\Schema(
     *             type="object",
     *             @SWG\Property(
     *                 property="success",
     *                 type="boolean",
     *                 description="Indica se a operação foi bem-sucedida"
     *             ),
     *             @SWG\Property(
     *                 property="data",
     *                 type="object",
     *                 @SWG\Property(
     *                     property="call_id",
     *                     type="integer",
     *                     description="ID da chamada criada"
     *                 )
     *             )
     *         )
     *     ),
     *     @SWG\Response(
     *         response=400,
     *         description="Dados inválidos"
     *     ),
     *     security={
     *         {"Bearer": {}}
     *     }
     * )
     */
    public function registerUserCall(Request $request)
    {
        Log::debug('Iniciando rotina de registro de ligação telefônica');
        $response = new Response();
        $params = $request->only(['car_id', 'comprador_id']);

        if (!$params['car_id'] || !$params['comprador_id']) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Faltam parâmetros para esta requisição']);
            return $response;
        }

        $car = Car::find($params['car_id']);
        if ($car) {
            if ($car->user_id !== $params['comprador_id']) {
                $call = Call::firstOrCreate($params);
                return ['success' => true, 'data' => ['call_id' => $call['id']]];
            }
        }

        return ['success' => false, 'msg' => 'Anúncio não existe ou anúncio pertence ao usuário'];

    }

    public function denounce(Request $request)
    {
        $response = new Response();

        //pegar os parâmetros
        $params = $request->only(['id', 'motivo', 'observacao']);

        //validar (obrigatório anúncio e motivo)
        $validation = Validator::make(
            $params,
            [
                'id' => ['required'],
                'motivo' => ['required']
            ],
            [
                'required' => 'O campo não foi preenchido',
            ]
        );

        if ($validation->fails()) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => $validation->messages()]);
            return $response;

        }

        // O anúncio existe?
        $anuncio = Car::find($params['id']);
        if (!$anuncio) {
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'Anúncio não localizado']);
            return $response;
        }
        //Dados do usuário denunciante (o usuário logado)
        $denunciante = Auth::user();


        //Dados anuncio
        $carUtil = new CarUtil($anuncio);
        $dadosAnuncio = $carUtil->getCarObject();
        $observation = ($params['observacao'] != null) ? $params['observacao'] : '';

        // Salvar denúncia
        $denuncia = CarIncident::create([
            'car_id' => $params['id'],
            'user_id' => $denunciante->id,
            'reason' => $params['motivo'],
            'observation' => $observation
        ]);

        $denuncia->status = 1;
        $denuncia->save();


        $job = (new SendEmailDenuncia($dadosAnuncio, $dadosAnuncio->user, $denunciante, $params['motivo'],
            $params['observacao']))->onQueue('emails');
        $this->dispatch($job);

        $admins = User::where('role_id', 1)->whereNotNull('device_token')->get();
        if ($admins) {
            $pushNotification = new PushNotification();
            $carIncident = CarIncident::where('status', 1)->count();

            $pushDenuncia = (new SendPushDenunciaAdmin($admins, $pushNotification, $carIncident))->onQueue('push');
            $this->dispatch($pushDenuncia);
        }


        $response->setStatusCode(200);
        $response->setContent(['succcess' => true, 'msg' => 'Denúncia efetuada com sucesso']);

        return $response;

    }

    public function passwordReset(Request $request)
    {
        $response = new Response();
        //checa se o campo foi preenchido
        if (!$request->has('email')) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Falta um parâmetro']);
            return $response;
        }

        $userSave = User::where('email', $request->input('email'))->first();
        //checa se usuário existe
        if ($userSave) {
            $token = random_bytes(32);
            $userSave->remember_token = bin2hex($token);
            //grava um token no banco
            $link = config('app.url') . '/recuperar-senha/' . bin2hex($token);
            if ($userSave->save()) {
                //enfileira e-mail de restauracao da senha

                $job = (new SendEmailSenha($userSave->email, $userSave->name, $link))->onQueue('emails');
                $this->dispatch($job);


            }
        }

        //Não retorna nenhum dado a resposta sempre vai ser essa:
        return ['success' => true, 'msg' => 'Instruções para o desbloqueio foram enviadas para seu e-mail'];
    }

    /**
     *
     * @SWG\Put(path="/store/{id}/migrate/cars",
     *   tags={"Store"},
     *   summary="Migrar anúncios de um usuário para outro",
     *   description="",
     *   operationId="carMigrate",
     *   produces={"application/json"},
     *   @SWG\Parameter( in="path", name="id", description="ID da Loja", required=true, type="integer", @SWG\Schema(ref="#/definitions/Store") ),
     *   @SWG\Parameter( in="formData", name="user_from", description="ID do usuário que vai ter os anúncios migrados", required=true, type="integer", @SWG\Schema(ref="#/definitions/User") ),
     *   @SWG\Parameter( in="formData", name="user_to", description="ID do usuário que vai receber os anúncios migrados", required=true, type="string", @SWG\Schema(ref="#/definitions/User") ),
     *   @SWG\Response( response=200, description="", @SWG\Schema(
     *            @SWG\Property( property="success", type="boolean", description="Migrado com sucesso" ),
     *            @SWG\Property( property="msg", type="string", description="Mensagem"),
     *        ) ),
     *   @SWG\Response( response=400, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="error", type="string", description="Bad Request" )
     *       )
     *   ),
     *   @SWG\Response( response=404, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="message", type="string", description="Model not found " ),
     *       )
     *   ),
     *   @SWG\Response( response=422, description="", @SWG\Schema(
     *            @SWG\Property( property="message", type="string", description="Validation failed" ),
     *            @SWG\Property( property="errors", type="array",
     *                @SWG\Items( type="object",
     *                    @SWG\Property(property="field", type="string"),
     *                )
     *            ),
     *        )
     *    ),
     * )
     * @param MigrateRequest $request
     * @param $store_id
     * @return \Illuminate\Http\JsonResponse
     */
    public function migrateCars(MigrateRequest $request, $store_id)
    {
        try {
            $user = auth()->user();
            $fields = $request->all();

            if (!$store = Store::find($store_id)) {
                return $this->modelNotFound([
                    'success' => false,
                    'msg' => trans('messages.error.notfound'),
                    'errors' => []
                ]);
            }

            if (!$user->hasRole('chief')) {
                return $this->badRequest([
                    'success' => false,
                    'msg' => trans('messages.error.forbidden'),
                    'errors' => []
                ], 401);
            }

            $belongs_store_user_from = $store->staff->contains(function ($key, $value) use ($fields) {
                return $value->id == $fields['user_from'];
            });

            $belongs_store_user_to = $store->staff->contains(function ($key, $value) use ($fields) {
                return $value->id == $fields['user_to'];
            });

            if (!$belongs_store_user_to || !$belongs_store_user_from) {
                return $this->badRequest([
                    'success' => false,
                    'msg' => trans('messages.error.store.staff.not_belongs'),
                    'errors' => []
                ], 400);
            }

            $user_from = User::find($fields['user_from']);

            if ($user_from->anuncios()->count() <= 0) {
                return $this->badRequest([
                    'success' => false,
                    'msg' => trans('messages.error.store.staff.no_have_cars'),
                    'errors' => []
                ], 400);
            }

            foreach ($user_from->anuncios()->where('store_id', $store->id)->get() as $_car) {
                $_car->fill(['user_id' => $fields['user_to']])->update();
            }

            return $this->ApiResponse(['success' => true, 'msg' => trans('messages.success.migrated')]);
        } catch (\Exception $e) {
            \Sentry::captureException($e);
            Log::error($e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine());
            return $this->badRequest(['success' => false, 'msg' => trans('messages.error.unavailable'), 'errors' => []],
                500);
        }
    }

}
