<?php
/**
 * Created by PhpStorm.
 * User: Rafael
 * Date: 05/09/2016
 * Time: 13:40
 */

namespace App\Chat;

use App\Chat\Pool\PoolFactory;
use App\Chat\Pool\PoolManagement;
use App\Chat\Room\RoomFactory;
use App\Core\Jobs\ChatNotification;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Ratchet\MessageComponentInterface;
use Ratchet\ConnectionInterface;
use App\Auth\Models\User;
use App\Admin\Dashboard\Models\Room as Sala;
use Log;
use Carbon\Carbon;
use Sentry;



class Chat implements MessageComponentInterface
{
    use DispatchesJobs;

    protected $clients;
    protected $admins;
    protected $client_status;

    public function __construct()
    {
        $this->clients = array();
        $this->admins = array();
        $this->client_status = array();

        $this->client_status = ClientStatus::boot();

    }

    /**
     * When a new connection is opened it will be passed to this method
     * @param  ConnectionInterface $conn The socket/connection that just connected to your application
     * @throws \Exception
     */
    public function onOpen(ConnectionInterface $conn)
    {
        // O token vem junto com o objeto request
        $token = $conn->WebSocket->request->getQuery()->get('token');
        // Auth é a classe responsável por validar a credencial (token)
        $auth = new Auth();
        $user = $auth->getCredentials($token);
        if (!$user) {
            // Token inválido fecha a conexão sem fazer nada
            Log::info('Sistema de Chat: Usuario não autenticado', ['token' => $token]);
            $conn->close();
        } else {
            $agora = Carbon::now()->toRfc3339String();
            // Usuário padrão entra no array clients e tem o status e a data do last_seen atualizada no array client_status
            $auth->setLastSeen($user['id']);
            $user['conn'] = $conn;
            $user['last_seen'] = $agora;
            if(isset($this->clients[$user['id']])) {
                Log::debug("Sistema de chat: Este usuário já estava conectado, conn resource anterior: {$this->clients[$user['id']]['conn']->resourceId} ");
            }
            $this->clients[$user['id']] = $user;

            $this->client_status[$user['id']]['last_seen'] = $agora;
            $this->client_status[$user['id']]['status'] = 'online';

            if ($user['role_id'] == 1) {
                Log::info('Sistema de Chat: Este usuário é administrador');
                echo "Um usuario administrativo está online {$user['id']}\n ";
                $this->admins[$user['id']] = $user;
            }
            Log::info("Sistema de Chat: Usuario Online: {$user['name']}, Id: {$user['id']}, Conn resource: ({$conn->resourceId})");
            echo "Usuario Online: {$user['name']}, Id: {$user['id']}, Conn resource: ({$conn->resourceId})\n";
            //Listar mensagens pendentes do pool

            $pool = new PoolManagement();
            $pooledMessages  = $pool->fetchUnsentByReceiver($user['id']);

            $messages= [];

            foreach ($pooledMessages as $pooledMessage) {
                $json = json_decode($pooledMessage['json'], true);

                $json['pool_id'] = $pooledMessage->id;
                $json['created_at'] = $pooledMessage->created_at->toRfc3339String();
                $json['delayed'] = 1;
                $messages[] = $json;
            }

            if (!empty($messages)) {
                $conn->send(json_encode($messages));
                Log::debug('Sistema de chat, grupo de mensagens offline recebidas: ' . json_encode($messages));
            }
            unset($pool, $pooledMessages, $messages);

            return ;
        }
    }

    /**
     * This is called before or after a socket is closed (depends on how it's closed).  SendMessage to $conn will not result in an error if it has already been closed.
     * @param  ConnectionInterface $conn The socket/connection that is closing/closed
     * @throws \Exception
     */
    public function onClose(ConnectionInterface $conn)
    {

        foreach ($this->clients as $key => $client) {
            if( $conn == $client['conn'] ) {

                $auth = new Auth();
                $auth->setLastSeen($key);
                $name = $this->clients[$key]['name'];

                $this->client_status[$key]['status'] = 'offline';
                $this->client_status[$key]['last_seen'] = Carbon::now()->toRfc3339String();

                unset($this->clients[$key]);

                echo "Usuario {$key} - {$name} {$conn->resourceId} desconectou\n";
                Log::info("Sistema de Chat: Usuario {$key} - {$name} {$conn->resourceId} desconectou");
            }
        }
    }

    /**
     * If there is an error with one of the sockets, or somewhere in the application where an Exception is thrown,
     * the Exception is sent back down the stack, handled by the Server and bubbled back up the application through this method
     * @param  ConnectionInterface $conn
     * @param  \Exception $e
     * @throws \Exception
     */
    public function onError(ConnectionInterface $conn, \Exception $e)
    {
        echo "An error has occurred: {$e->getMessage()}\n";
        Log::error('Sistema de Chat: Erro encontrado: ' . $e->getMessage(), ['error' => $e ] );
        $erro_cliente = [
            'error' => true,
            'msg' => $e->getMessage()
        ];
        $conn->send(json_encode($erro_cliente));
        Sentry::captureException($e);
//        $conn->close();
    }

    /**
     * Triggered when a client sends data through the socket
     * @param  \Ratchet\ConnectionInterface $from The socket/connection that sent the message to your application
     * @param  string $json The message received
     * @throws \Exception
     */
    public function onMessage(ConnectionInterface $from, $json)
    {

        Log::debug("Sistema de chat mensagem enviada no canal {$from->resourceId}:" . $json);

        $msg = json_decode($json, true);

        // Dos tipos de mensagens  (online heartbeat composing)

        if(isset($msg['heartbeat']) and $msg['heartbeat'] == 1 ) {
            $msg['quantidade'] = count($this->clients);
            $from->send(json_encode($msg));
            unset($msg);
            return ;
        }

        $ack = (isset($msg['ack']) AND $msg['ack'] == 1) ? true : false;

        if ($ack) {
            if(empty($msg['pool_ids'])) {
                throw new \Exception('Parâmetro pool_ids não informado!');
            }

            $pool = new PoolManagement();

            foreach($msg['pool_ids'] as $pool_id) {
                $pool->setMessageSent($pool_id);
            }
            unset($msg, $pool);
            return ;
        }

        $online = ( isset($msg['online']) AND $msg['online'] == 1 ) ? true : false;

        if ($online) {
            //Toda vez que você pede o status de outro usuário o seu próprio status é atualizado
            $saveUser = $this->setClientLastSeen($msg['sender_id']);
            $userInfo = $this->getClientStatus($msg['receiver_id']);

            $msg['last_seen'] = $userInfo['last_seen'];

            $tempo = $this->timeDiff($msg['last_seen']);

            if ( $tempo > 5 ) {
                // O usuário está cinco MINUTOS sem entrar num chat nesse caso ele provavelmente caiu
                $msg['status'] = 'offline';
                // Mais pra frente dá pra derrubar esse cara do socket nesse momento

            } else {
                $msg['status'] = $userInfo['status'];
            }

            $from->send(json_encode($msg));

            unset($msg, $userInfo, $saveUser);

            return ;

        }

        // Ticket#178 - marcar o carro como vendido ou excluir gera essa mensagem e faz o chat enviar uma mensagem pra cada usuário que tem conversa com ele


        if ( isset($msg['car_status_id']) ) {
            $msg['stopBroadcast'] = true;
            // Pool do remetente
            $senderMsg = $msg;
            $senderMsg['room_id'] = 0;
            $senderMsg['receiver_id'] = (int)$msg['sender_id'];
            $senderMsg['sender_id'] = null;
            $senderPool = PoolFactory::create($msg['sender_id'], 6, $senderMsg);
            $senderMsg['pool_id'] = $senderPool['pool_id'];
            if ( isset( $this->clients[$msg['sender_id']] ) ) {

                $this->clients[$msg['sender_id']]['conn']->send(json_encode($senderMsg));
            }
            // listar todos os usuários com conversa iniciada
            $car_id = $msg['car_id'];
            $interessados = Sala::where('car_id', $car_id)->get(['id', 'comprador_id'])->toArray();

            foreach ($interessados as $interessado) {
                $msg['receiver_id'] = (int)$interessado['comprador_id'];
                $msg['room_id'] = $interessado['id'];
                $pool = PoolFactory::create($interessado['comprador_id'], 4, $msg);
                $msg['pool_id'] = $pool['pool_id'];

                if ( isset($this->clients[$interessado['comprador_id']]) ) {
                    $this->clients[$interessado['comprador_id']]['conn']->send(json_encode($msg));
                }
            }




            unset($msg, $interessados, $car_id, $pool, $senderMsg);
            return ;
        }

        //Ticket#179 se o sender_id for igual ao receiver_id retornar um erro
        if(isset($msg['sender_id'], $msg['receiver_id'])) {
            if ($msg['sender_id'] == $msg['receiver_id']) {
                Log::error("Sistema de Chat: Erro encontrado: Remetente é igual ao destinatário Remetente: {$msg['sender_id']} Destinatário: {$msg['receiver_id']}");
                $resposta = [
                    'car_id' => $msg['car_id'],
                    'error' => true,
                    'msg' => 'O remetente e o destinatário da mensagem são o mesmo'
                ];

                $from->send(json_encode($resposta));

                unset($msg, $resposta);

                return ;
            }
        }


        // Se a mensagem for composing ou recording não pegar sala nem salvar no banco, apenas enviar a mensagem
        $composing = ( isset($msg['typing']) OR isset($msg['recording']) ) ? true : false;

        if ($composing) {
            if(isset($this->clients[$msg['receiver_id']])) {
                $this->clients[$msg['receiver_id']]['conn']->send(json_encode($msg));
            }

            unset($msg);
            return ;
        }

        // Se a mensagem for um recibo notificação de leitura:
        $recibo = (isset($msg['read']) AND $msg['read'] == 1) ? true : false;

        $user_ack = (isset($msg['received']) AND $msg['received'] == 1) ? true : false;

        if ($recibo OR $user_ack) {
            /* Aqui a mensagem é do tipo recibo de mensagem (chegou no aparelho)
             * ou do tipo recibo de leitura (usuário abriu o chat)
             * O sinal da mensagem será invertido (sender e receiver) e vai gerar pool da mensagem e enviar
             * além disso é incluído stopBroadcast para que essa mensagem não transite novamente no chat
             * (ou seja elas se encerram ao serem recebidas no aparelho)
             */
            if($recibo) {
                $msg_type = 3;
            }
            if($user_ack) {
                $msg_type = 2;
            }

            // Se a mensagem for um recibo, significa que é uma mensagem que tem o sender e o receiver trocados (sender é o receiver)
            // Essa mensagem vai de B para A fazendo pool caso A esteja offline . Pool vai não gera ack para esse tipo de mensagem
            $receiver = $msg['sender_id'];
            $sender = $msg['receiver_id'];

            $msg['stopBroadcast'] = true;

            if ( isset($msg_type) ) {
                $pool = PoolFactory::create($receiver, $msg_type, $msg);
                $msg['pool_id'] = $pool['pool_id'];
            }

            if ( isset($this->clients[$receiver]) ) {
                $this->clients[$receiver]['conn']->send(json_encode($msg));
                return ;
            }

            unset($msg, $pool, $receiver, $sender, $msg_type);
            return ;
        }

       /*
        * Conforme documentação do chat esta função foi unida a de recibo por terem os mesmos passps
        * inverte o sinal (sender_id receiver_id) coloca stopBroadcast e faz pool e manda pro cliente
        *
        if ($user_ack) {
            $receiver = $msg['sender_id'];
            $sender = $msg['receiver_id'];
            $pool = PoolFactory::create($receiver, 2, $msg);
            $msg['pool_id'] = $pool['pool_id'];
            if ( isset($this->clients[$receiver]) ) {
                $this->clients[$receiver]['conn']->send(json_encode($msg));
                return ;
            }

            unset($msg, $pool, $receiver, $sender);
            return ;

        }
       */

        // Mensagem padrão (conversa)

        //Criar sala (caso não exista)
        if (!isset($msg['room_id'])) {
            try {
                $room = RoomFactory::create($msg['car_id'], $msg['sender_id']);
                $room_id = $room->getRoomId();
            } catch (\Exception $e) {
                \Sentry::captureException($e);
                Log::error("Sistema de Chat Erro encontrado: {$e->getMessage()}");
                $msg['error'] = true;
                $msg['msg'] = $e->getMessage();
                $msg['car_status_id'] = isset($room) ? $room->getCarStatus() : 0 ;
                $from->send(json_encode($msg));
                return ;
            }
            $msg['room_id'] = $room_id;
        }

        $pool = PoolFactory::create($msg['receiver_id'], 1, $msg);

        // Salvar ID e Data da cadastro na mensagem

        $msg['created_at'] = $pool['created_at'];
        $msg['received'] = 0;
        $msg['stopBroadcast'] = true;
        $poolSender = PoolFactory::create($msg['sender_id'], 5, $msg);
        $msg['pool_id'] = $poolSender['pool_id'];
        $from->send(json_encode($msg));
        unset($msg['stopBroadcast']);

        $msg['pool_id'] = $pool['pool_id'];
        // Novo envio
        //@todo a tabela mensagem não existe mais se der tempo remover o push daqui e colocar no cron
        if ( isset($this->clients[$msg['receiver_id']]) ) {
            $this->clients[$msg['receiver_id']]['conn']->send(json_encode($msg));
            return ;
        } else {

            $receiverData = User::find($msg['receiver_id']);

            if ($receiverData && $receiverData->device_token) {
                if ( empty($msg['message']) ) {
                    if ( !empty($msg['image'])) {
                        $texto = "\xf0\x9f\x93\xb7 Foto enviada";
                    } else {
                        $texto = "\xf0\x9f\x8e\xa4 Mensagem de voz";
                    }
                } else {
                    $texto = $msg['message'];
                }

                $parameters = [
                    'include_player_ids' => [
                        $receiverData->device_token
                    ],
                    'headings' => [
                        'en' => $this->clients[$msg['sender_id']]['name']
                    ],
                    'contents' => [
                        'en' => $texto
                    ],
                    'android_group' => 1,
                    'android_group_message' => [
                        'en' => 'Você tem $[notif_count] novas mensagens'
                    ],
                    'small_icon' => 'repasse_small_icon_white',
                    'large_icon' => 'repasse_icon',
                ];

                $job = ( new ChatNotification( $parameters ) )->onQueue('push');
                $this->dispatch($job);
            }
            return ;
        }
    }

    private function getClientStatus($user_id) {
        return $this->client_status[$user_id];
    }

    private function setClientLastSeen($user_id) {
        $date = Carbon::now()->toRfc3339String();
        $this->client_status[$user_id]['last_seen'] = $date;
        return $date;
    }

    private function timeDiff($timeB) {
        $timeA = Carbon::now();
        $timeB = Carbon::createFromFormat(DATE_RFC3339, $timeB);

        return $timeB->diffInMinutes($timeA);

    }

}