<?php

namespace App\Admin\Dashboard\Models;

use App\Repasse\TimeParser;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use App\Auth\Models\User;
use App\Admin\Dashboard\Models\Store\Store;
use App\Admin\Dashboard\Models\HistoricCarTurbo;

/**
 * @SWG\Definition(type="object", @SWG\Xml(name="Car"))
 */
class Car extends Model
{
    protected $table = 'car';

    protected $fillable = ['podio_id', 'color_id', 'doors', 'eixos', 'brand_id', 'fipe_id', 'year', 'observation', 'cost', 'price', 'km', 'store_id', 'user_id'];

    protected $dates = ['created_at', 'updated_at', 'boosted_until'];

    protected $hidden = ['updated_at'];

    // protected $appends = ['fipe_percent_off'];

    protected $casts = [
        'cost' => 'double'
    ];

    protected $expirar = 60; //Dias

    public static function boot()
    {
        parent::boot();


        static::deleting(function ($car) {
            $car->photos()->delete();
            $car->optionals()->delete();

        });
    }

    public function getCreatedAtAttribute($value)
    {
        return TimeParser::format($value);
    }

    public function getPublishedAtAttribute($value)
    {
        return TimeParser::format($value);
    }

    public function getFipeoffAttribute()
    {
        $fipe_percent_off = ($this->price - $this->fipe_price) / $this->fipe_price * 100;
        return number_format($fipe_percent_off) . '%';
    }

    public function getExpiradoAttribute()
    {
        $dias = $this->expirar;
        $data = Carbon::now()->subDays($dias);
        $dataAnuncio = Carbon::createFromFormat(DATE_RFC3339, $this->created_at);


        return $dataAnuncio->lte($data);

    }

    public function user()
    {
        return $this->belongsTo(User::class, 'user_id');
    }

    public function fipe()
    {
        return $this->belongsTo(Fipe::class, 'fipe_id');
    }

    public function color()
    {
        return $this->belongsTo(Color::class, 'color_id');
    }

    public function brand()
    {
        return $this->belongsTo(Brand::class, 'brand_id');
    }

    public function photos()
    {
        return $this->hasMany(CarPhoto::class)->orderByRaw('COALESCE(photo_order, 10) ASC');
    }

    public function optionals()
    {
        return $this->hasMany(CarOptional::class);
    }

    public function pushes()
    {
        return $this->hasMany(CarPush::class);
    }

    public function incidents()
    {
        return $this->hasMany(CarIncident::class);
    }

    public function moderator()
    {
        return $this->hasMany(CarModeration::class);
    }

    public function car_status()
    {
        return $this->hasOne(CarStatus::class, 'id', 'car_status_id');
    }

    public function car_sell()
    {
        return $this->hasOne(CarSell::class);
    }

    public function rooms()
    {
        return $this->hasMany(Room::class)->orderBy('bid', 'desc');
    }

    public function calls()
    {
        return $this->hasMany(Call::class);
    }

    public function store()
    {
        return $this->hasOne( Store::class, 'id', 'store_id' );
    }

    public function turbos()
    {
        return $this->hasMany( HistoricCarTurbo::class, 'car_id' );
    }

    public function scopeWhereLikeModel( $query, $model )
    {
        if ( !is_null( $model ) ) {
            return $query->leftJoin( 'fipe', 'car.fipe_id', '=', 'fipe.id' )->where( 'fipe.model', 'like', '%' . $model . '%' );
        }

        return $query;
    }

    public function scopeCarStatusAtivo($query)
    {
        return $query->whereIn('car_status_id', [1,2,9]);
    }
}
