<?php

namespace App\Api\Http\Controllers;

use App\Admin\Dashboard\Models\City;
use App\Admin\Dashboard\Models\State;
use App\Core\Http\Controllers\CoreController;
use App\Repasse\GeoLocation;
use App\Repasse\Integration\Cep\BrasilApi\BrasilApiFetch;
use App\Repasse\Integration\Cep\CepFetchInterface;
use App\Repasse\Integration\Cep\DTO\CepDTO;
use App\Repasse\Integration\Cep\DTO\FetchCepDTO;
use App\Repasse\Integration\Cep\Geocoder\GeocoderCepFetch;
use App\Repasse\Integration\Cep\OpenCEP\OpenCepFetch;
use App\Repasse\Integration\Cep\ViaCep\ViaCepFetch;
use Geocoder\Laravel\Facades\Geocoder;
use Geocoder\Query\GeocodeQuery;
use GuzzleHttp\Exception\ClientException;
use Illuminate\Support\Facades\Log;

class ApiCepController extends CoreController
{

    /**
     * @SWG\Get(path="/cep/{cep}",
     *   tags={"Cep"},
     *   summary="Buscar um endereço através do CEP",
     *   description="",
     *   operationId="populateModel",
     *   produces={"application/json"},
     *    @SWG\Parameter( in="path", name="cep", description="CEP do endereço", required=true, type="integer" ),
     *   @SWG\Response( response=200, description="",
     *       @SWG\Schema(
     *           @SWG\Property(property="success", type="boolean"),
     *           @SWG\Property(property="data", type="object", ref="#/definitions/CepDTO"
     *           )
     *       )
     *   ),
     *   @SWG\Response( response=400, description="",
     *       @SWG\Schema(
     *           @SWG\Property( property="error", type="string", description="Bad Request" )
     *       )
     *   )
     * )
     * @param $cep
     * @param OpenCepFetch $viaCepFetch
     * @return \Illuminate\Http\Response
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function getCep($cep, CepFetchInterface $openCepFetch)
    {
        //
        try {
            // Log::info('opa');
            $cepData = $openCepFetch->fetchCep($cep);
        } catch (ClientException $e) {
            Log::error('Erro no serviço de CEP', [$e]);
            $statusCode = $e->getResponse()->getStatusCode();
            if ($statusCode === 404) {
                return $this->returnError($statusCode, "O CEP informado {$cep} nao foi encontrado");
            }
            return $this->returnError(500, $e->getMessage());
        }
        // com os dados do CEP, procurar cidade e estado e incluir no retorno
        $state = State::where('initials', '=', $cepData->getEstado())
            ->firstOrFail();
        $city = City::where('stateId', '=', $state->id)
            ->where('name', '=', $cepData->getCidade())
            ->firstOrFail();

        $cep = new \App\Repasse\Integration\Cep\DTO\CepDTO($cepData, $city, $state, get_class($openCepFetch));

        return $this->returnData($cep);

    }

    public function getCepGeocoder($cep, GeocoderCepFetch $geocoderFetch)
    {
        $result = $geocoderFetch->fetchCep($cep);
        if ($result) {
            $state = State::where('initials', '=', $result->uf)->firstOrFail();
            $city = City::where('stateId', '=', $state->id)->where('name', '=', $result->localidade)->firstOrFail();
            $cepDto = new CepDTO($result, $city, $state);

            return $this->returnData($cepDto);
        }
        return $this->returnError(404, "O CEP informado {$cep} nao foi encontrado");

    }

    public function getCepBrasilApi($cep, BrasilApiFetch $fetch)
    {
        try {
            /** @var FetchCepDTO $cepData */
            $cepData = $fetch->fetchCep($cep);
        } catch (ClientException $e) {
            dd($e);
            return $this->returnError(404, "O CEP informado {$cep} nao foi encontrado");
        }
        $state = State::where('initials', '=', $cepData->getEstado())->firstOrFail();
        $city = City::where('stateId', '=', $state->id)
            ->where('name', '=', $cepData->getCidade())
            ->firstOrFail();
        return $this->returnData( new CepDTO($cepData, $city, $state));


    }

}