<?php

namespace App\Api\Http\Controllers\Checkout;

use DB;
use Moip;
use App\Auth\Models\User;
use Illuminate\Support\Facades\Log;
use App\Traits\Rest\ResponseHelpers;
use App\Admin\Dashboard\Models\Product;
use App\Admin\Dashboard\Models\Order\Order;
use App\Http\Requests\Checkout\CheckoutRequest;
use App\Admin\Dashboard\Models\CarQuery\CarQuery;
use App\Admin\Dashboard\Models\OrderItem\OrderItem;
use App\Core\Http\Controllers\CoreController as CoreController;

class CheckoutController extends CoreController
{
	use ResponseHelpers;

	/**
    * @SWG\Post(path="/checkout",
    *    tags={"Checkout"}, summary="Finish Checkout", description="", operationId="finishCheckout", produces={"application/json"},
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *    @SWG\Parameter( name="product_id", in="formData", description="", required=true, type="integer" ),
    *    @SWG\Parameter( name="license_plate", in="formData",  description="Max: 7", required=true, type="string" ),
    *    @SWG\Parameter( name="dob", in="formData",  description="Date of birthday - Format: Y-m-d", required=true, type="string" ),
    *    @SWG\Parameter( name="credit_card_number", in="formData",  description="", required=true, type="string" ),
    *    @SWG\Parameter( name="credit_card_date", in="formData",  description="", required=true, type="string" ),
    *    @SWG\Parameter( name="credit_validation_code", in="formData",  description="", required=true, type="string" ),
    *    @SWG\Response( response=201, description="", @SWG\Schema( ref="#/definitions/City" ) ),
    *    @SWG\Response( response=400, description="", @SWG\Schema( @SWG\Property( property="error", type="string", description="Bad Request") ) ),
    *    @SWG\Response( response=401, description="", @SWG\Schema(
    *            @SWG\Property( property="error", type="string", description="Error message" )
    *        )
    *    ),
    *    @SWG\Response( response=404, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Model not found " ),
    *        )
    *    ),
    *    @SWG\Response( response=422, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Validation failed" ),
    *            @SWG\Property( property="errors", type="array",
    *                @SWG\Items( type="object",
    *                    @SWG\Property(property="field", type="string"),
    *                )
    *            ),
    *        )
    *    ),
    * )
    */
    public function checkout( CheckoutRequest $request )
    {
        try {
        	$fields  = $request->all();
        	$user    = auth()->user();
        	$moip    = Moip::start();
        	$product = Product::find( $fields['product_id'] );
        	$cred_card_date = explode( '/', $fields['credit_card_date'] );

        	if ( is_null( $user->cell_phone ) ) {
        		return $this->badRequest([
                    'success' => false,
                    'msg' => 'Encontramos alguns erros.',
        			'errors'=>[
        				'cell_phone' => [ trans( 'messages.error.cell_phone.null' ) ]
        			]
    			] , 422 );
        	}

        	DB::beginTransaction();

        	$order = Order::create([
        		'user_id'  => $user->id,
        		'total'    => ( $product->price / 100 ),
        		'subtotal' => ( $product->price / 100 )
        	]);

        	OrderItem::create([
        		'order_id'   => $order->id,
        		'product_id' => $product->id,
        		'price'      => ( $product->price / 100 ),
        		'qty'        => 1,
        		'options'    => json_encode( ['license_plate'=>$fields['license_plate']] )
        	]);

        	$customer = $moip->customers()->setOwnId( uniqid() )
			        		 ->setFullname( $user->name )
			        		 ->setEmail( $user->email )
			        		 ->setBirthDate( $fields['dob'] )
			        		 ->setTaxDocument( $user->document )
			        		 ->setPhone( substr( $user->cell_phone, 0, 2 ), substr( $user->cell_phone, 2 ) )
					         ->addAddress(
					        	'BILLING',
					            $user->address, $user->number,
					            $user->neighborhood, $user->city, $user->state,
					            $user->zipcode )
					         ->addAddress(
					        	'SHIPPING',
					            $user->address, $user->number,
					            $user->neighborhood, $user->city, $user->state,
					            $user->zipcode )
					         ->create();

	        $order_moip = $moip->orders()->setOwnId( uniqid() )
						       ->addItem( $product->name, 1, $product->id, $product->price )
						       ->setCustomer( $customer )
						       ->create();

			$payment = $order_moip->payments()->setCreditCard(
							$cred_card_date[0],
							$cred_card_date[1],
							$fields['credit_card_number'],
							$fields['credit_validation_code'],
							$customer
						)->execute();

			$order->fill( ['transaction_code'=>$payment->getOrder()->getId()] )->save();

			DB::commit();

			return $this->ApiResponse( ['success'=>true, 'msg'=>trans( 'messages.success.checkout.finished' )], 201 );
        } catch ( \Moip\Exceptions\UnautorizedException $e ) {
            \Sentry::captureException($e);
        	//StatusCode 401
        	Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
         	return $this->badRequest( ['success'=>false, 'msg'=>$e->getMessage()] , 500 );
		} catch ( \Moip\Exceptions\ValidationException $e ) {
            \Sentry::captureException($e);
		    //StatusCode entre 400 e 499 (exceto 401)
		    DB::rollBack();
        	Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
         	return $this->badRequest( ['success'=>false, 'msg'=>$e->__toString()] , 500 );
		} catch ( \Moip\Exceptions\UnexpectedException $e ) {
            \Sentry::captureException($e);
		    //StatusCode >= 500
		    DB::rollBack();
        	Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
         	return $this->badRequest( ['success'=>true, 'msg'=>trans( 'messages.error.unavailable' )] , 500 );
		}
    }
}
