<?php

namespace App\Api\Http\Controllers\Order;

use DB;
use Moip;
use App\Auth\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use App\Traits\Rest\ResponseHelpers;
use Illuminate\Pagination\Paginator;
use App\Admin\Dashboard\Models\Product;
use App\Admin\Dashboard\Models\Order\Order;
use App\Transformers\Order\OrderTransformer;
use App\Admin\Dashboard\Models\CarQuery\CarQuery;
use App\Admin\Dashboard\Models\OrderItem\OrderItem;
use App\Core\Http\Controllers\CoreController as CoreController;

class OrderController extends CoreController
{
	use ResponseHelpers;

    /**
    * @SWG\Get(path="/orders",
    *   tags={"Order"},
    *   summary="List All Orders",
    *   description="",
    *   operationId="listOrders",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="query", name="per_page", description="Default: 20 | Use -1 for get all registers", required=false, type="integer" ),
    *   @SWG\Parameter( in="query", name="current_page", description="Default: 1", required=false, type="integer" ),
    *   @SWG\Parameter( in="query", name="order", description="ASC or DESC | Default: ASC", required=false, type="string" ),
    *   @SWG\Parameter( in="query", name="order_by", description="options: ID or created_at | Default:ID", required=false, type="string" ),
    *   @SWG\Parameter( in="query", name="status", description="0=Aguardando Pagamento, 1=Pagamento Aprovado, 2=Pedido Cancelado, 3=Falha no Pagamento", required=false, type="integer" ),
    *   @SWG\Parameter( in="query", name="grouped", description="Enviar true para agrupar o resultado por placa. Quando é agrupado por placa não existe paginação", required=false, type="integer" ),
    *   @SWG\Response( response=200, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="status_code", type="integer", description="200" ),
    *           @SWG\Property( property="data", type="array",
    *               @SWG\Items( type="object",
    *                   @SWG\Property(property="totals", type="integer"),
    *                   @SWG\Property(property="per_page", type="integer"),
    *                   @SWG\Property(property="current_page", type="integer"),
    *                   @SWG\Property(property="items", type="array", @SWG\Items(
    *                       type="object", ref="#/definitions/Order"
    *                   ))
    *               )
    *           ),
    *       )
    *   ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   )
    * )
    */
    public function getAllByUserOrders( Request $request )
    {
    	try {
    		$user = auth()->user();

    		$per_page      = $request->input('per_page') ? $request->input('per_page') : 10;
	        $current_page  = $request->input('current_page') ? $request->input('current_page') : 1;
	        $order_by      = $request->input('order_by') ? $request->input('order_by') : 'created_at';
            $order         = $request->input('order') ? $request->input('order') : 'DESC';
            $status        = $request->input('status') != '' ? [$request->input('status')] : [0,1,2,3];
            $grouped       = $request->input('grouped') ? $request->input('grouped') : null;

	        Paginator::currentPageResolver( function() use ( $current_page ) {
	            return $current_page;
	        });

	        $orders = $user->orders()->join( 'car_queries', 'orders.id', '=', 'car_queries.order_id' )
                                     ->orderBy( 'orders.' . $order_by, $order )
	        						 ->whereIn( 'orders.status', $status )
                                     ->select( 'orders.*' )
	        						 ->paginate( $per_page );

            if ( !is_null( $grouped ) ) {
                $orders = $user->orders()->join( 'car_queries', 'orders.id', '=', 'car_queries.order_id' )
                                     ->orderBy( 'orders.' . $order_by, $order )
                                     ->whereIn( 'orders.status', $status )
                                     ->select( 'orders.*' )
                                     ->get();

                $orders_grouped = $orders->groupBy( 'car_query.license_plate' );
		        return ( new OrderTransformer )->transformGroupedCollection( $orders_grouped );
            }

            return ( new OrderTransformer )->transformCollection( $orders );
    	} catch ( \Exception $e ) {
            \Sentry::captureException($e);
			Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
         	return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
    	}
    }

    /**
    * @SWG\Get(path="/order/{id}",
    *   tags={"Order"},
    *   summary="View a Order",
    *   description="",
    *   operationId="viewOrder",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="path", name="id", description="", required=true, type="integer" ),
    *   @SWG\Response( response=200, description="", @SWG\Schema(ref="#/definitions/Order") ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   ),
    *   @SWG\Response( response=404, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="message", type="string", description="Model not found " ),
    *       )
    *   ),
    * )
    */
    public function viewOrder( $id )
    {
    	try {
    		$user = auth()->user();

    		if ( !$order = $user->orders()->where( 'id', $id )->first() ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            return ( new OrderTransformer )->transform( $order );
    	} catch ( \Exception $e ) {
            \Sentry::captureException($e);
    		Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
         	return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
    	}
    }
}
