<?php

namespace App\Api\Http\Controllers\Plan;

use DB;
use App\Auth\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use App\Traits\Rest\ResponseHelpers;
use Illuminate\Pagination\Paginator;
use App\Admin\Dashboard\Models\Plan\Plan;
use App\Http\Requests\Plan\CreateRequest;
use App\Http\Requests\Plan\UpdateRequest;
use App\Transformers\Plan\PlanTransformer;
use App\Core\Http\Controllers\CoreController as CoreController;

class PlanController extends CoreController
{
	use ResponseHelpers;

    /**
    * @SWG\Post(path="/plan",
    *    tags={"Plan"}, summary="Create Plan", description="", operationId="createPlan", produces={"application/json"},
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *    @SWG\Parameter( name="title", in="formData", description="Min Lenght: 4 | Max Lenght: 255 ", required=true, type="string" ),
    *    @SWG\Parameter( name="price", in="formData",  description="", required=true, type="integer" ),
    *    @SWG\Parameter( name="trial_period", in="formData",  description="", required=false, type="integer" ),
    *    @SWG\Parameter( name="transaction_price", in="formData",  description="", required=true, type="integer" ),
    *    @SWG\Parameter( name="qty_turbos", in="formData",  description="", required=true, type="integer" ),
    *    @SWG\Parameter( name="qty_car_queries", in="formData",  description="", required=true, type="integer" ),
    *    @SWG\Parameter( name="qty_transactions", in="formData",  description="", required=true, type="integer" ),
    *    @SWG\Parameter( name="status", in="formData",  description="0=Disabled, 1=Enabled", required=true, type="integer" ),
    *    @SWG\Response( response=201, description="", @SWG\Schema( ref="#/definitions/Plan" ) ),
    *    @SWG\Response( response=400, description="", @SWG\Schema( @SWG\Property( property="error", type="string", description="Bad Request") ) ),
    *    @SWG\Response( response=401, description="", @SWG\Schema(
    *            @SWG\Property( property="error", type="string", description="Error message" )
    *        )
    *    ),
    *    @SWG\Response( response=404, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Model not found " ),
    *        )
    *    ),
    *    @SWG\Response( response=422, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Validation failed" ),
    *            @SWG\Property( property="errors", type="array",
    *                @SWG\Items( type="object",
    *                    @SWG\Property(property="field", type="string"),
    *                )
    *            ),
    *        )
    *    ),
    * )
    */
    public function create( CreateRequest $request )
    {
        try {
            $fields = $request->all();
            $slug   = str_slug( $fields['title'] );
            if ( $fields['transaction_price'] > $fields['price'] ) {
                return $this->badRequest([
                    'success' => false,
                    'msg' => 'Encontramos alguns erros.',
                    'errors'=>[
                        'transaction_price' => [ trans( 'messages.error.transaction_price.expensive' ) ]
                    ]
                ] , 422 );
            }

            $fields['slug'] = $slug;
            if ( $exist_slug = Plan::where( 'slug', $slug )->first() ) {
                $extra = 1;
                $slug = $slug . '-' . $extra;
                while( Plan::whereSlug( $slug )->exists() ) {
                    $extra++;
                    $slug = preg_replace( '/[0-9]+/', '', $slug ) . $extra;
                }
                $fields['slug'] = $slug;
            }

            $plan = Plan::create( $fields );

            return ( new PlanTransformer )->transform( $plan );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Get(path="/plans",
    *   tags={"Plan"},
    *   summary="List All plans",
    *   description="",
    *   operationId="listPlans",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="query", name="per_page", description="Default: 20", required=false, type="integer" ),
    *   @SWG\Parameter( in="query", name="current_page", description="Default: 1", required=false, type="integer" ),
    *   @SWG\Parameter( in="query", name="order", description="ASC or DESC | Default: ASC", required=false, type="string" ),
    *   @SWG\Parameter( in="query", name="order_by", description="options: ID or created_at | Default:ID", required=false, type="string" ),
    *   @SWG\Parameter( in="query", name="status", description="0=Disabled, 1=Enabled", required=false, type="integer" ),
    *   @SWG\Response( response=200, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="status_code", type="integer", description="200" ),
    *           @SWG\Property( property="data", type="array",
    *               @SWG\Items( type="object",
    *                   @SWG\Property(property="totals", type="integer"),
    *                   @SWG\Property(property="per_page", type="integer"),
    *                   @SWG\Property(property="current_page", type="integer"),
    *                   @SWG\Property(property="items", type="array", @SWG\Items(
    *                       type="object", ref="#/definitions/Plan"
    *                   ))
    *               )
    *           ),
    *       )
    *   ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   )
    * )
    */
    public function index( Request $request )
    {
        try {
            $per_page      = $request->input('per_page') ? $request->input('per_page') : 10;
            $current_page  = $request->input('current_page') ? $request->input('current_page') : 1;
            $order_by      = $request->input('order_by') ? $request->input('order_by') : 'created_at';
            $order         = $request->input('order') ? $request->input('order') : 'DESC';
            $status        = $request->input('status') != '' ? [$request->input('status')] : [0,1];

            Paginator::currentPageResolver( function() use ( $current_page ) {
                return $current_page;
            });

            $plans = Plan::orderBy( $order_by, $order )->whereIn( 'status', $status )->paginate( $per_page );

            return ( new PlanTransformer )->transformCollection( $plans );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Get(path="/plan/{id}",
    *   tags={"Plan"},
    *   summary="View a Plan",
    *   description="",
    *   operationId="viewPlan",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="path", name="id", description="", required=true, type="integer" ),
    *   @SWG\Response( response=200, description="", @SWG\Schema(ref="#/definitions/Plan") ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   ),
    *   @SWG\Response( response=404, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="message", type="string", description="Model not found " ),
    *       )
    *   ),
    * )
    */
    public function show( $id )
    {
        try {
            if ( !$plan = Plan::find( $id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            return ( new PlanTransformer )->transform( $plan );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Put(path="/plan/{id}",
    *    tags={"Plan"}, summary="Update Plan", description="", operationId="updatePlan", produces={"application/json"},
    *    @SWG\Parameter( in="header", name="Authorization", description="Basic Auth", required=true, type="string" ),
    *    @SWG\Parameter( name="id", in="query", description="", required=true, type="integer" ),
    *    @SWG\Parameter( name="title", in="formData", description="Min Lenght: 4 | Max Lenght: 255 ", required=false, type="string" ),
    *    @SWG\Parameter( name="price", in="formData",  description="", required=false, type="integer" ),
    *    @SWG\Parameter( name="trial_period", in="formData",  description="", required=false, type="integer" ),
    *    @SWG\Parameter( name="transaction_price", in="formData",  description="", required=false, type="integer" ),
    *    @SWG\Parameter( name="qty_turbos", in="formData",  description="", required=false, type="integer" ),
    *    @SWG\Parameter( name="qty_car_queries", in="formData",  description="", required=false, type="integer" ),
    *    @SWG\Parameter( name="qty_transactions", in="formData",  description="", required=false, type="integer" ),
    *    @SWG\Parameter( name="status", in="formData",  description="0=Disabled, 1=Enabled", required=false, type="integer" ),
    *    @SWG\Response( response=201, description="", @SWG\Schema( ref="#/definitions/Plan" ) ),
    *    @SWG\Response( response=400, description="", @SWG\Schema( @SWG\Property( property="error", type="string", description="Bad Request") ) ),
    *    @SWG\Response( response=401, description="", @SWG\Schema(
    *            @SWG\Property( property="error", type="string", description="Error message" )
    *        )
    *    ),
    *    @SWG\Response( response=404, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Model not found " ),
    *        )
    *    ),
    *    @SWG\Response( response=422, description="", @SWG\Schema(
    *            @SWG\Property( property="message", type="string", description="Validation failed" ),
    *            @SWG\Property( property="errors", type="array",
    *                @SWG\Items( type="object",
    *                    @SWG\Property(property="field", type="string"),
    *                )
    *            ),
    *        )
    *    ),
    * )
    */
    public function update( UpdateRequest $request, $id )
    {
        try {
            if ( !$plan = Plan::find( $id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            $fields = $request->all();

            if ( isset( $fields['transaction_price'], $fields['price'] ) ) {
                if ( $fields['transaction_price'] > $fields['price'] ) {
                    return $this->badRequest([
                        'success' => false,
                        'msg' => 'Encontramos alguns erros.',
                        'errors'=>[
                            'transaction_price' => [ trans( 'messages.error.transaction_price.expensive' ) ]
                        ]
                    ] , 422 );
                }
            }

            if ( isset( $fields['transaction_price'] ) && !isset( $fields['price'] ) ) {
                if ( $fields['transaction_price'] > $plan->transaction_price ) {
                    return $this->badRequest([
                        'success' => false,
                        'msg' => 'Encontramos alguns erros.',
                        'errors'=>[
                            'transaction_price' => [ trans( 'messages.error.transaction_price.expensive' ) ]
                        ]
                    ] , 422 );
                }
            }

            if ( isset( $fields['title'] ) ) {
                $slug = str_slug( $fields['title'] );
                $fields['slug'] = $slug;
                if ( $exist_slug = Plan::where( 'slug', $slug )->first() ) {
                    $extra = 1;
                    $slug = $slug . '-' . $extra;
                    while( Plan::whereSlug( $slug )->exists() ) {
                        $extra++;
                        $slug = preg_replace( '/[0-9]+/', '', $slug ) . $extra;
                    }
                    $fields['slug'] = $slug;
                }
            }

            $plan->update( $fields );

            return ( new PlanTransformer )->transform( $plan );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Delete(path="/plan/{id}",
    *   tags={"Plan"},
    *   summary="Delete a Plan",
    *   description="",
    *   operationId="deletePlan",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="path", name="id", description="", required=true, type="integer" ),
    *   @SWG\Response( response=204, description="Ok but not content" ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   ),
    *   @SWG\Response( response=404, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="message", type="string", description="Model not found " ),
    *       )
    *   ),
    * )
    */
    public function delete( $id )
    {
        try {
            if ( !$plan = Plan::find( $id ) ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
            }

            if ( $plan->stores()->count() > 0 ) {
                return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.plan.not_remove' ), 'errors'=>[]] , 400 );
            }

            $plan->delete();

            return $this->ApiResponse( ['success'=>true, 'msg'=>trans( 'messages.success.removed' )] );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }
}
