<?php
/**
 * Created by PhpStorm.
 * User: Rafael
 * Date: 24/10/2016
 * Time: 19:09
 */

namespace App\Api\Http\Controllers;

use App\Admin\Dashboard\Models\State;
use App\Auth\Models\User;
use Carbon\Carbon;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Auth;
use App\Admin\Dashboard\Models\UserProfile;
use App\Admin\Dashboard\Models\UserProfileCarType;
use App\Admin\Dashboard\Models\UserProfileDDD;
use App\Core\Http\Controllers\CoreController;
use Illuminate\Http\Request;
use Illuminate\Http\Response;


class ProfileController extends CoreController
{
    /** @var UserProfile $userProfile */
    protected $userProfile;

    /** @var  Auth $authUser */
    protected $authUser;

    /** @var Request $request */
    protected $request;

    /**
     * ProfileController constructor.
     * @param UserProfile $userProfile
     * @param Auth $auth
     */
    public function __construct(UserProfile $userProfile, Auth $auth, Request $request) {
        $this->userProfile = $userProfile;
        $this->authUser = $auth::user();
        $this->request = $request;
    }

    /**
     * Sets data from profile
     * @param $profileObject
     * @return mixed
     */
    private function getProfileData($profileObject) {

        /** @var UserProfileDDD $dddPivot */
        $dddPivot = UserProfileDDD::where('profile_id', $profileObject->id)->get();

        $ddd = [];
        foreach ($dddPivot as  $singleDDD) {
            $query = 'select d.id, d.state_id, s.name as state_name, d.ddd as ddd, d.name from state_ddds d JOIN states s ON s.id = d.state_id WHERE d.id = :ddd_id ;';

            $queryParameters = [
                'ddd_id' => $singleDDD->ddd_id
            ];
            $dddData = \DB::select($query, $queryParameters);
            $ddd[] = $dddData[0];
        }

        $profileObject->ddds = $ddd;

        // type vehicle data

        $types = [];

        /** @var UserProfileCarType $carPivot */
        $carPivot = UserProfileCarType::where('profile_id', $profileObject->id)->get(['car_type']);

        foreach ($carPivot as $cpv) {
            $query = 'select t.id, t.name from car_types t where t.id = :tid;';

            $queryParameters = [
                'tid' => $cpv->car_type
            ];
            $typeData = \DB::select($query, $queryParameters);
            $types[] = $typeData[0];
        }

        $profileObject->types = $types;

        return $profileObject;

    }

    public function getProfileStates() {
        $profile_states = $this->userProfile->where('user_id', $this->authUser->id)->get(['state_id'])->toArray();

        $states = State::whereNotIn('id', $profile_states)->get(['id', 'initials', 'name']);

        return ['success' => true, 'data' => $states];
    }


    /**
     * Get a single profile
     * @param null $profile_id
     * @return array|Response
     * @throws \InvalidArgumentException
     */
    public function getProfile($profile_id = null) {
        $response = new Response();

        if (null === $profile_id) {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Perfil não informado']);
            return $response;
        }

        $profileObject = $this->userProfile->find($profile_id);

        if (null === $profileObject) {
            $response->setStatusCode(404);
            $response->setContent(['success' => false, 'msg' => 'O Perfil informado não existe']);
            return $response;
        }

        if ($this->authUser->id !== $profileObject->user_id) {
            $response->setStatusCode(403);
            $response->setContent(['success' => false, 'msg' => 'O Perfil informado não pertence ao seu usuário']);
            return $response;
        }


        return ['success' => true, 'data' => $this->getProfileData($profileObject)];
    }

    public function saveProfile() {


        $requestData = $this->request->all();

        $validation = Validator::make(
            $requestData,
            [
                'state_id' => ['required'],
                'array_ddd' => ['required'],
                'array_types' => ['required'],
                'start_year' => ['required'],
                'end_year' => ['required']
            ],
            [
                'required' => 'O campo precisa ser preenchido.',
            ]
        );

        if ($validation->fails()) {
            $response = new Response();
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => $validation->messages()]);
            return $response;

        }

        $requestData['user_id'] = $this->authUser->id;

        /** @noinspection StaticInvocationViaThisInspection */
        $profile = $this->userProfile->create($requestData);
        $profile->ddds()->sync($requestData['array_ddd']);
        $profile->types()->sync($requestData['array_types']);
        $profile->save();

        return ['success' => true, 'data' => $this->getProfileData($profile) ];
    }


    public function updateProfile( $profile_id) {

        $requestData = $this->request->all();

        $validation = Validator::make(
            $requestData,
            [
                'state_id' => ['required'],
                'array_ddd' => ['required'],
                'array_types' => ['required'],
                'start_year' => ['required'],
                'end_year' => ['required']
            ],
            [
                'required' => 'O campo precisa ser preenchido.',
            ]
        );

        if ($validation->fails()) {
            $response = new Response();
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => $validation->messages()]);
            return $response;

        }

        $this->userProfile->find($profile_id)->update($requestData);

        $profile = $this->userProfile->find($profile_id);
        $profile->ddds()->sync($requestData['array_ddd']);
        $profile->types()->sync($requestData['array_types']);
        $profile->save();

        return ['success' => true, 'data' => $this->getProfileData($profile) ];

    }

    public function listProfile() {
        $profiles = $this->userProfile->where('user_id', $this->authUser->id)->get();

        $returnedProfiles = [];

        foreach($profiles as $profile) {
            $returnedProfiles[] = $this->getProfileData($profile);
        }

        return ['success' => true, 'data' => $returnedProfiles];
    }

    public function removeProfile() {
        $response = new Response();

        if ($this->request->has('id')) {
            $profiles_id = $this->request->input('id');

            $profiles = $this->userProfile->find($profiles_id);

            if (count($profiles) != count($profiles_id)) {
                $response->setStatusCode(404);
                $response->setContent(['success' => false, 'msg' => 'Um ou mais perfis informados não existe']);
                return $response;
            }

            // Verificar se os perfis informados são deste usuário
            foreach ($profiles as $profile) {
                if($profile->user_id != $this->authUser->id) {
                    $response->setStatusCode(403);
                    $response->setContent(['success' => false, 'msg' => 'Acesso negado']);
                    return $response;
                }
            }

            $deleteProfiles = $this->userProfile->destroy($profiles_id);

            return ['success' => true, 'msg' => 'Perfi(s) removido(s) com sucesso!'];

        } else {
            $response->setStatusCode(400);
            $response->setContent(['success' => false, 'msg' => 'Perfil não informado']);
            return $response;
        }
    }

    public function muteProfile() {
        $now = Carbon::now();

        switch($this->request->input('time')) {
            case '2h':
                $now->addHours(2);
                break;
            case '8h':
                $now->addHours(8);
                break;
            case '1d':
                $now->addHours(24);
                break;
            case '2d':
                $now->addHours(48);
                break;
            case '1s':
                $now->addWeek(1);
                break;
        }

        $user = User::find($this->authUser->id);
        $user->muted_until = $now;
        $user->save();

        return ['success' => true, 'data' => ['muted_until' => $now->toRfc3339String()]];
    }

    public function mutedUntil() {
        $user = User::find($this->authUser->id);
        return ['success' => true, 'data' => ['muted_until' => $user->muted_until->toRfc3339String() ]  ];

    }

    public function mutedList() {
        $mutedOptions = [
            0 => [
                'label' => 'Duas Horas',
                'value' => '2h'
            ],
            1 => [
                'label' => 'Oito Horas',
                'value' => '8h'
            ],
            2 => [
                'label' => 'Um Dia (24 horas)',
                'value' => '1d'
            ],
            3 => [
                'label' => 'Dois Dias (48 Horas)',
                'value' => '2d'
            ],
            4 => [
                'label' => 'Uma Semana',
                'value' => '1s'
            ],
        ];

        return ['success' => true, 'data' => ['muted_options' => $mutedOptions]];

    }

    public function removeMuted() {
        $now = Carbon::now();
        $user = User::find($this->authUser->id);
        $user->muted_until = $now;
        $user->save();

        return ['success' => true, 'data' => ['muted_until' => $now->toRfc3339String()]];

    }

}