<?php

namespace App\Api\Http\Controllers\Report;

use App\Admin\Dashboard\Models\CarTurbo;
use App\Admin\Dashboard\Models\Product;
use DB;
use Carbon\Carbon;
use App\Auth\Models\User;
use Illuminate\Http\Request;
use App\Admin\Dashboard\Models\Car;
use Illuminate\Support\Facades\Log;
use App\Admin\Dashboard\Models\Room;
use Illuminate\Pagination\Paginator;
use App\Traits\Rest\ResponseHelpers;
use App\Admin\Dashboard\Models\Store\Store;
use App\Admin\Dashboard\Models\Transaction;
use App\Http\Requests\Report\BillingRequest;
use App\Http\Requests\Report\OverviewRequest;
use App\Admin\Dashboard\Models\HistoricCarTurbo;
use App\Core\Http\Controllers\CoreController as CoreController;

class ReportController extends CoreController
{
	use ResponseHelpers;

    /**
    * @SWG\Get(path="/report/overview",
    *   tags={"Report"},
    *   summary="View the report",
    *   description="",
    *   operationId="viewReportOverview",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="path", name="id", description="ID do pedido", required=true, type="string", @SWG\Schema(ref="#/definitions/Order") ),
    *   @SWG\Response( response=200, description="", @SWG\Schema(ref="#/definitions/CarQuery") ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   ),
    *   @SWG\Response( response=404, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="message", type="string", description="Model not found " ),
    *       )
    *   ),
    * )
    */
	public function index( OverviewRequest $request )
	{
		try {
            $fields = $request->all();
            $user   = auth()->user();
            $data   = [];
            $hoje = Carbon::now();
            $expirado = Carbon::now()->subDays(60);

            // Verificação para saber se o usuário é realmente o diretor da loja.
            if ( !$store = $user->stores()->where( 'store_user.store_id', $fields['store_id'] )->where( 'store_user.role_id', 2 )->first() ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]], 401 );
            }

            $staff = $store->staff->pluck('id')->toArray();

            /**
             * 1 = Em aberto
             * 2 = Negociando
             * 3 = Vendido
             * 4 = Em cadastro
             * 5 = Em analise
             * 6 = Suspenso
             * 7 = Bloqueado
             * 8 = Excluido
             * 9 = Em Aberto
             */

            /**
             * 1 = Últimos 7 dias
             * 2 = Mês atual (do começo do mês ao dia atual)
             * 3 = Último mês
             * 4 = Último trimestre
             * 5 = Último semestre
             * 6 = Último ano
             */

            $date_range = ( isset( $fields['date_range'] ) ? $fields['date_range'] : 1 );
            switch ( $date_range ) {
                case 1:
                    $start_date = Carbon::now()->subDays( 6 )->format( 'Y-m-d H:i:s' );
                    $end_date   = Carbon::now()->format( 'Y-m-d H:i:s' );
                    break;
                case 2:
                    $start_date = Carbon::now()->format( 'Y-m-01 H:i:s' );
                    $end_date   = Carbon::now()->format( 'Y-m-d H:i:s' );
                    break;
                case 3:
                    $start_date = Carbon::now()->subMonths(1)->format( 'Y-m-01 00:00:00' );
                    $end_date   = Carbon::now()->subMonths(1)->format( 'Y-m-t 23:59:59' );
                    break;
                case 4:
                    $start_date = Carbon::now()->subMonths(2)->format( 'Y-m-01 H:i:s' );
                    $end_date   = Carbon::now()->format( 'Y-m-t H:i:s' );
                    break;
                case 5:
                    $start_date = Carbon::now()->subMonths(5)->format( 'Y-m-01 H:i:s' );
                    $end_date   = Carbon::now()->format( 'Y-m-d H:i:s' );
                    break;
                case 6:
                    $start_date = Carbon::now()->subYear()->format( 'Y-m-d H:i:s' );
                    $end_date   = Carbon::now()->format( 'Y-m-d H:i:s' );
                    break;
                default:
                    $start_date = Carbon::now()->subDays( 6 )->format( 'Y-m-d H:i:s' );
                    $end_date   = Carbon::now()->format( 'Y-m-d H:i:s' );
                    break;
            }

            // Total anunciado
            $total_announced = Car::whereIn( 'user_id', $staff )->whereBetween( 'created_at', [$start_date, $end_date] )->whereIn( 'car_status_id', [1,2,3,9] )->sum( 'price' );
            // Total Vendido
            $total_sold      = Car::whereIn( 'user_id', $staff )->whereBetween( 'created_at', [$start_date, $end_date] )->where( 'car_status_id', 3 )->sum( 'price' );
            // Total disponíveis para venda (o anúncio expira em sessenta dias e tem que estar com o status em aberto e não faz diferença o período selecionado
            // $total_available = Car::whereIn( 'user_id', $staff )->whereBetween( 'created_at', [$start_date, $end_date] )->whereIn( 'car_status_id', [1,2,9] )->sum( 'price' );
            $total_available = Car::whereIn( 'user_id', $staff )->whereBetween( 'created_at', [$expirado->format('Y-m-d H:i:s'), $hoje->format('Y-m-d H:i:s')] )->whereIn( 'car_status_id', [1,2,9] )->sum( 'price' );


            // Quantidade Total de anunciados na plataforma
            $qty_announced       = Car::whereIn( 'user_id', $staff )->whereBetween( 'created_at', [$start_date, $end_date] )->whereIn( 'car_status_id', [1,2,3,9] )->count();
            // Quantidade total de anuncios negociados na plataforma
            //$qty_negotiated      = Car::whereIn( 'user_id', $staff )->whereBetween( 'created_at', [$start_date, $end_date] )->where( 'car_status_id', 2 )->count();
            // Calls received (on my ads):
            $qty_received_calls = (int)Car::join('calls', 'calls.car_id', '=', 'car.id')->where('car.user_id', $user->id)->whereBetween( 'calls.created_at', [$start_date, $end_date] )->count();

            // Calls sent on other ads
            $qty_my_calls = (int)$user->calls()->whereBetween( 'created_at', [$start_date, $end_date] )->count();

            // Chats received on my ads (fiz o join manual pq são duas tabelas e o laravel não tava deixando)
            $qty_received_chats = (int)Car::join( 'rooms', 'rooms.car_id', '=', 'car.id' )->whereIn( 'car.user_id', $staff )->whereBetween( 'rooms.created_at', [$start_date, $end_date] )->count();

            // Chats started on other ads
            $qty_my_chats = Room::whereIn( 'comprador_id', $staff )->whereBetween( 'created_at', [$start_date, $end_date] )->count();

            //qty negotiated é a soma de todos eles
            $qty_negotiated = ($qty_received_calls + $qty_my_calls + $qty_received_chats + $qty_my_chats);

            // Quantidade total de anuncios vendidos na plataforma
            $qty_sold            = Car::whereIn( 'user_id', $staff )->whereBetween( 'created_at', [$start_date, $end_date] )->where( 'car_status_id', 3 )->count();
            // Quantidade total de anuncios disponível na plataforma
            $qty_available       = Car::whereIn( 'user_id', $staff )->whereBetween( 'created_at',  [$expirado->format('Y-m-d H:i:s'), $hoje->format('Y-m-d H:i:s')] )->whereIn( 'car_status_id', [1,2,9] )->count();

            // Quantidade total de anuncios com turbo ativo
            $total_turbos_active = Car::whereIn( 'user_id', $staff )->whereBetween( 'published_at', [$start_date, $end_date] )->where( 'car_status_id', 9 )->count();

            $total_turbos_used = $user->turbos()->count();

            $data['overview'] = [
                'total_announced'     => $total_announced,
                'total_sold'          => $total_sold,
                'total_available'     => $total_available,
                'qty_announced'       => $qty_announced,
                'qty_negotiated'      => $qty_negotiated,
                'qty_sold'            => $qty_sold,
                'qty_available'       => $qty_available,
                'total_turbos_active' => $total_turbos_active,
                'turbos_used'         => $total_turbos_used
            ];

            $cars_sold      = Car::whereIn( 'user_id', $staff )->whereBetween( 'created_at', [$start_date, $end_date] )->where( 'car_status_id', 3 )->get();
            $cars_announced = Car::whereIn( 'user_id', $staff )->whereBetween( 'created_at', [$start_date, $end_date] )->whereIn( 'car_status_id', [1,2,3,9] )->get();
            $cars = Car::where('store_id', $store->id)->whereBetween( 'created_at', [$start_date, $end_date])->lists('id')->toArray() ;
            //$cars_turbos    = Car::whereIn( 'user_id', $staff )->whereBetween( 'published_at', [$start_date, $end_date] )->where( 'car_status_id', 9 )->get();
            $cars_turbos = CarTurbo::whereIn('user_id', $staff)->whereIn('car_id', $cars)->get();

            $format_date = 'Y-m-d';
            if ( in_array( $date_range, [4,5,6] ) ) {
                $format_date = 'Y-m';
            }

            $cars_sold_grouped = $cars_sold->groupBy( function( $val ) use( $format_date ) {
                return Carbon::parse( $val->created_at )->format( $format_date );
            });

            $cars_announced_grouped = $cars_announced->groupBy( function( $val ) use( $format_date ) {
                return Carbon::parse( $val->created_at )->format( $format_date );
            });

            $cars_turbos_grouped = $cars_turbos->groupBy( function( $val ) use( $format_date ) {
                return Carbon::parse( $val->created_at )->format( $format_date );
            });

            $values_cars_sold = $prices_sold_agrouped = [];
            foreach ( $cars_sold_grouped as $key => $value ) {
                $values_cars_sold[$key] = count( $value );

                foreach ( $value as $val ) {
                    $prices_sold_agrouped[$key][] = $val->price;
                }
            }

            $values_cars_announced = [];
            foreach ( $cars_announced_grouped as $key => $value ) {
                $values_cars_announced[$key] = count( $value );
            }

            $values_cars_turbos = [];
            foreach ( $cars_turbos_grouped as $key => $value ) {
                $values_cars_turbos[$key] = count( $value );
            }

            $axis_x = [];
            if ( in_array( $date_range, [1,2,3] ) ) {
                while ( $start_date <= $end_date ) {
                    $axis_x[]   = date( 'Y-m-d', strtotime( $start_date ) );
                    $start_date = date( 'Y-m-d H:i:s', strtotime( "+1 day", strtotime( $start_date ) ) );
                }
            } else {
                while ( $start_date <= $end_date ) {
                    $axis_x[]   = date( 'Y-m', strtotime( $start_date ) );
                    $start_date = date( 'Y-m-d H:i:s', strtotime( "+1 month", strtotime( $start_date ) ) );
                }
            }

            $values_announced = $values_sold = $values_turbos = $prices_sold = [];
            foreach ( $axis_x as $value ) {
                if ( array_key_exists( $value, $values_cars_announced ) ) {
                    $values_announced[] = $values_cars_announced[$value];
                } else {
                    $values_announced[] = 0;
                }

                if ( array_key_exists( $value, $values_cars_sold ) ) {
                    $values_sold[] = $values_cars_sold[$value];
                } else {
                    $values_sold[] = 0;
                }

                if ( array_key_exists( $value, $values_cars_turbos ) ) {
                    $values_turbos[] = $values_cars_turbos[$value];
                } else {
                    $values_turbos[] = 0;
                }

                if ( array_key_exists( $value, $prices_sold_agrouped ) ) {
                    $prices_sold[] = array_sum( $prices_sold_agrouped[$value] );
                } else {
                    $prices_sold[] = 0;
                }
            }

            $graphic_one = [
                'axis_x' => $axis_x,
                'values' => [
                    $values_announced,
                    $values_sold,
                    $values_turbos
                ]
            ];

            $graphic_two = [
                'axis_x' => $axis_x,
                'values' => [
                    $prices_sold
                ]
            ];

            $average_sold      = ( $total_sold / count( $axis_x ) );
            $avarage_availabel = ( $total_available / count( $axis_x ) );

            $data['graphic_one'] = $graphic_one;
            $data['graphic_two'] = $graphic_two;
            $data['average'] = [
                'average_sold'      => $average_sold,
                'avarage_availabel' => $avarage_availabel,
                'total_sold'        => $total_sold
            ];

            return $this->ApiResponse( ['success'=>true, 'data'=>$data] );

		} catch ( \Exception $e ) {
            \Sentry::captureException($e);
			Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
         	return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
		}
	}

    /**
    * @SWG\Get(path="/report/scoreboard",
    *   tags={"Report"},
    *   summary="View mini score",
    *   description="",
    *   operationId="viewMiniScore",
    *   produces={"application/json"},
    *   @SWG\Response( response=200, description="Dados do Scoreboard", @SWG\Schema(type="object",
    *            @SWG\Property(property="success", type="boolean"),
    *            @SWG\Property(property="data", type="object",
    *                  @SWG\Property(property="qty_active", type="integer"),
    *                  @SWG\Property(property="qty_salesman", type="integer"),
    *                  @SWG\Property(property="qty_cars", type="integer"),
    *                  @SWG\Property(property="qty_transactions", type="integer"),
    *                  @SWG\Property(property="qty_turbos", type="integer"),
    *            )
    *     )
    *    ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   ),
    *   @SWG\Response( response=404, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="message", type="string", description="Model not found " ),
    *       )
    *   ),
    * )
    */
    public function scoreboard()
    {
        try {
            $user  = auth()->user();
            $start = Carbon::now()->startOfMonth()->format( 'Y-m-d 00:00:00' );
            $end   = Carbon::now()->format( 'Y-m-d H:i:s' );
            $hoje = Carbon::now();
            $expirado = Carbon::now()->subDays(60);
            $store = $user->myStore();

            if ( !(bool)$store ) {
                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]], 404 );
            }

            $salesman = $store->staff()->where( 'store_user.role_id', 4 )->where( 'store_user.status', 1 );
            $qty_salesman     = $salesman->count();

            // Alterado para considerar os anúncios publicados no mês (published_at not 00-00-0000)
            $cars = Car::where( 'store_id', $store->id )->whereRaw('published_at > 0')->whereNotIn('car_status_id', [4]);
            $turbo = clone($cars);
            $qty_turbos = $turbo->where('car_status_id', 9)->count(); // CarTurbo::whereIn('user_id', $salesman_list)->whereIn('car_id', $cars_list)->whereBetween('created_at', [$start, $end])->count();
            $qty_cars         = $cars->whereBetween( 'created_at', [$start, $end] )->count();
            $qty_transactions = $store->transactions()->whereBetween( 'created_at', [$start, $end] )->count();
            // $qty_turbos       = $user->turbos()->whereBetween( 'created_at', [$start, $end] )->count();
            // Alterado para trazer os dados da loja ao invés do usuário único que usou o turbo, mostrando anúncios que o diretor turbinou pro vendedor (por exemplo)
            // $qty_turbos = CarTurbo::whereIn('user_id', $salesman_list)->whereIn('car_id', $cars_list)->whereBetween('created_at', [$start, $end])->count();
            $qty_active = Car::where( 'store_id', $store->id )->whereIn( 'car_status_id', [1,2,9] )->count();

            //$qty_franchise = $user->turbos()->whereBetween( 'created_at', [$start, $end] )->count();
            $staff = $store->staff()->lists('id')->toArray();
            $qty_franchise = HistoricCarTurbo::whereIn('user_id', $staff)->whereBetween( 'created_at', [$start, $end] )->count();

            $data = [
                'qty_active'       => $qty_active,
                'qty_salesman'     => $qty_salesman,
                'qty_cars'         => $qty_cars,
                'qty_transactions' => $qty_transactions,
                'qty_turbos'       => $qty_turbos,
                'qty_franchise'    => $qty_franchise,
            ];

            return $this->ApiResponse( ['success'=>true, 'data'=>$data] );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    /**
    * @SWG\Get(path="/report/billing/{user_id?}",
    *   tags={"Report"},
    *   summary="View billing",
    *   description="",
    *   operationId="viewBilling",
    *   produces={"application/json"},
    *   @SWG\Parameter( in="path", name="user_id", description="ID do Usuário", required=false, type="string" ),
    *   @SWG\Parameter( in="path", name="start_date", description="Data inicial | Formato: Y-m-d", required=false, type="string" ),
    *   @SWG\Parameter( in="path", name="end_date", description="Data final | Formato: Y-m-d", required=false, type="string" ),
    *   @SWG\Response( response=200, description="", @SWG\Schema(ref="#/definitions/CarQuery") ),
    *   @SWG\Response( response=400, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="error", type="string", description="Bad Request" )
    *       )
    *   ),
    *   @SWG\Response( response=404, description="",
    *       @SWG\Schema(
    *           @SWG\Property( property="message", type="string", description="Model not found " ),
    *       )
    *   ),
    * )
    */
    public function getBilling( BillingRequest $request, $user_id = null )
    {
        try {
            $user  = auth()->user();

            if ( !is_null( $user_id ) ) {
                $user = User::find( $user_id );
            }

            if ( !$user->hasRole( ['chief', 'admin'] ) ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.forbidden' ), 'errors'=>[]] , 401 );
            }


            $start = $request->input('start_date') ? $request->input('start_date') : Carbon::now()->startOfMonth()->format( 'Y-m-d 00:00:00' );
            $end   = $request->input('end_date') ? $request->input('end_date') : Carbon::now()->format( 'Y-m-d H:i:s' );
            $store = $user->myStore();
            $staff = $store->staff()->lists('id')->toArray();
            $turbo_config = Product::where('type_id', 1)->where('status', 1)->first();


            if ( !$plan = $store->plans()->where( 'plan_store.status', 1 )->first() ) {
                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.plan_notfound' ), 'errors'=>[]] , 400 );
            }

            $info_plan = [
                'name'              => $plan->title,
                'price'             => (float)$plan->price,
                'transaction_price' => (float)$plan->transaction_price,
                'qty_turbos'        => $plan->qty_turbos,
                'qty_car_queries'   => $plan->qty_car_queries,
                'qty_transactions'  => $plan->qty_transactions
            ];

            $transactions = $store->transactions()->whereBetween( DB::raw(' CAST(created_at AS DATE)'), [$start, $end] )->get();

            $historic_transactions = [];
            foreach ( $transactions as $_transaction ) {
                $historic_transactions[] = [
                    'user'    => [
                        'id'    => $_transaction->user->id,
                        'name'  => $_transaction->user->name,
                        'email' => $_transaction->user->email,
                    ],
                    'car' => [
                        'id'    => $_transaction->car->id,
                        'price' => $_transaction->car->price,
                        'km'    => $_transaction->car->km,
                    ],
                    'type'       => $_transaction->type,
                    'created_at' => $_transaction->created_at->format( 'Y-m-d H:i:s' )
                ];
            }

            // $turbos = $user->turbos()->whereBetween( 'created_at', [$start, $end] )->get();
            $turbos = HistoricCarTurbo::whereIn('user_id', $staff)->whereBetween( DB::raw(' CAST(created_at AS DATE)'), [$start, $end] )->get();

            $historic_turbos = [];
            foreach ( $turbos as $_turbo ) {
                $historic_turbos[] = [
                    'id' => $_turbo->id,
                    'user'    => [
                        'id'    => $_turbo->user->id,
                        'name'  => $_turbo->user->name,
                        'email' => $_turbo->user->email,
                    ],
                    'car' => [
                        'id'    => $_turbo->car->id,
                        'price' => $_turbo->car->price,
                        'km'    => $_turbo->car->km,
                    ],
                    'created_at' => $_turbo->created_at->format( 'Y-m-d H:i:s' )
                ];
            }

            $car_queries = $user->carQueries()->whereBetween( 'created_at', [$start, $end] )->get();

            $historic_car_queries = [];
            foreach ( $car_queries as $_car_query ) {
                $historic_car_queries[] = [
                    'id' => $_car_query->id,
                    'user'    => [
                        'id'    => $_car_query->user->id,
                        'name'  => $_car_query->user->name,
                        'email' => $_car_query->user->email,
                    ],
                    'car_query' => [
                        'id'            => $_car_query->carQuery->id,
                        'order_id'      => $_car_query->carQuery->order_id,
                        'license_plate' => $_car_query->carQuery->license_plate,
                    ],
                    'created_at' => $_car_query->created_at->format( 'Y-m-d H:i:s' )
                ];
            }

            $qty_transactions = count( $historic_transactions );
            $qty_turbos       = count( $historic_turbos );
            $qty_queries      = count( $historic_car_queries );


            $amount_to_be_paid = 0;

            $transactions_extra = 0;
            if ( $qty_transactions > $plan->qty_transactions ) {
                $transactions_extra = $qty_transactions - $plan->qty_transactions;
            }

            $amount_transactions_extra = $transactions_extra * $plan->transaction_price;

            $turbos_extras = 0;
            if ( $qty_turbos > $plan->qty_turbos ) {
                $turbos_extras = $qty_turbos - $plan->qty_turbos;
            }

            $amount_turbos_extra = $turbos_extras * ($turbo_config->price / 100);

            $car_queries_extra = 0;
            if ( $qty_queries > $plan->qty_car_queries ) {
                $car_queries_extra = $qty_queries - $plan->qty_car_queries;
            }

            $amount_queries_extra = $car_queries_extra * $plan->transaction_price;

            $data = [
                'plan'                 => $info_plan,
                'transactions'         => $historic_transactions,
                'turbos'               => $historic_turbos,
                'historic_car_queries' => $historic_car_queries,
                'qty_transactions'     => $qty_transactions,
                'qty_turbos'           => $qty_turbos,
                'qty_queries'          => $qty_queries,
                'extra_charges'        => [
                    'amount_transactions_extra' => (double)$amount_transactions_extra,
                    'transactions_extra'        => $transactions_extra,
                    'amount_turbos_extra'       => (double)$amount_turbos_extra,
                    'turbos_extras'             => $turbos_extras,
                    'amount_queries_extra'      => (double)$amount_queries_extra,
                    'car_queries_extra'         => $car_queries_extra,
                    'total'                     => ( $plan->price + $amount_transactions_extra + $amount_turbos_extra + $amount_queries_extra )
                ],
                'start_date' => $start,
                'end_date'   => $end
            ];

            return $this->ApiResponse( ['success'=>true, 'data'=>$data] );
        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }
}
