<?php

namespace App\Api\Http\Controllers\Site;

use Illuminate\Support\Facades\Log;
use Illuminate\Support\Str;
use App\Traits\Rest\ResponseHelpers;
use App\Admin\Dashboard\Models\UserSite\Site;
use App\Core\Http\Controllers\CoreController as CoreController;
use App\Http\Requests\Site\SiteCreateRequest;
use App\Http\Requests\Site\SiteUpdateRequest;
use App\Transformers\Site\SiteTransformer;
use Illuminate\Http\Request;

class SiteController extends CoreController
{
	use ResponseHelpers;

	protected $user;

	public function __construct()
    {
        $this->user = auth()->user();
    }

    public function getByUser()
    {
        $site = Site::where( 'user_id', $this->user->id )->first();

        if ( !$site ) {

            return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );
        }

        $result = $this->verifySiteStatus($site);

        if ( $result['status'] == false ) {

            return $result['response'];
        }

        return ( new SiteTransformer )->transform( $site );
    }

    public function store( SiteCreateRequest $request )
    {
        try {
            $fields = $request->all();

            $site = Site::where( 'user_id', $this->user->id )->first();

            if ( $site ) {

                return $this->badRequest( ['success'=>false,'msg'=>trans( 'messages.error.site.exists' ), 'errors'=>[]] , 401 );
            }

            $fields['user_id'] = $this->user->id;
            $fields['token'] = Str::random(40);

            if ( $request->has('domain') ) {
                $fields['domain_status'] = Site::PENDING;
            }

            $site = Site::create( $fields );

            return ( new SiteTransformer )->transform( $site );


            
         } catch ( \Exception $e ) {

            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    public function update( SiteUpdateRequest $request )
    {
        try {
            $fields = $request->except(['user_id', 'status', 'token']);

            $site = Site::where( 'user_id', $this->user->id )->first();

            if ( !$site ) {

                return $this->modelNotFound( ['success'=>false, 'msg'=>trans( 'messages.error.notfound' ), 'errors'=>[]] );

            }
               
            $site->update( $fields );

            return ( new SiteTransformer )->transform( $site );

            
         } catch ( \Exception $e ) {

            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest( ['success'=>false, 'msg'=>trans( 'messages.error.unavailable' ), 'errors'=>[]] , 500 );
        }
    }

    public function getSiteBySlug( Request $request )
    {
        try {
            
            $site = Site::where( 'slug', $request->slug )->first();

            if ( !$site ) {

                return $this->modelNotFound([
                    'success'=>false,
                    'msg'=>trans( 'messages.error.notfound' ),
                    'errors'=>[]
                ]);
            }

            $result = $this->verifySiteStatus($site);

            if ( $result['status'] == false ) {

                return $result['response'];
            }

            return ( new SiteTransformer )->transform( $site );

        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest([
                'success'=>false,
                'msg'=>trans( 'messages.error.unavailable' ),
                'errors'=>[]], 500 
            );
        }
    }

    public function getSiteByToken( Request $request )
    {
        try {
            
            $site = Site::where( 'token', $request->token )->first();

            if ( !$site ) {
                return $this->modelNotFound([
                    'success'=>false,
                    'msg'=>trans( 'messages.error.notfound' ),
                    'errors'=>[]
                ]);
            }

            $result = $this->verifySiteStatus($site);

            if ( $result['status'] == false ) {

                return $result['response'];
            }

            return ( new SiteTransformer )->transform( $site );

        } catch ( \Exception $e ) {
            \Sentry::captureException($e);
            Log::error( $e->getMessage() . ' on file ' . $e->getFile() . ' on line ' . $e->getLine() );
            return $this->badRequest([
                'success'=>false,
                'msg'=>trans( 'messages.error.unavailable' ),
                'errors'=>[]], 500 
            );
        }
    }

    private function verifySiteStatus( Site $site )
    {
        if ( $site->registration_status == false ) {

            return [
                'status' => false,
                'response' => $this->forbidden( ['success'=>false, 'msg'=>trans( 'messages.error.site.registration' ), 'errors'=>[]] )
            ];
        }

        if ( $site->status == false ) {

            return [
                'status' => false,
                'response' => $this->forbidden( ['success'=>false, 'msg'=>trans( 'messages.error.user.pending' ), 'errors'=>[]] )
            ];
        }

        return ['status' => true];
    }
}